"""Verify if call to function or method inside tests are missing parentheses."""
# pylint: disable=using-constant-test, missing-docstring
# pylint: disable=invalid-name, expression-not-assigned, unnecessary-lambda-assignment

import os


import collections
import random

try:
    import multiprocessing
    from multiprocessing import synchronize  # noqa pylint: disable=unused-import
except ImportError:
    multiprocessing = None  # type: ignore[assignment]


def bool_function():
    return True


def nonbool_function():
    return 42


class Class:
    @staticmethod
    def bool_method():
        return False

    @staticmethod
    def nonbool_method():
        return 42


if collections:
    pass

if bool_function:  # [missing-parentheses-for-call-in-test]
    pass

if not bool_function():
    pass

if nonbool_function:  # [missing-parentheses-for-call-in-test]
    pass

if nonbool_function() != 42:
    pass

instance = Class()

if instance.bool_method:  # [missing-parentheses-for-call-in-test]
    pass

if not instance.bool_method():
    pass

if not instance.nonbool_method:
    pass
elif instance.bool_method:  # [missing-parentheses-for-call-in-test]
    pass

bool_lambda = lambda: True

if bool_lambda:  # [missing-parentheses-for-call-in-test]
    pass

if not bool_lambda():
    pass

nonbool_lambda = lambda: 42

if nonbool_lambda:  # [missing-parentheses-for-call-in-test]
    pass

if not nonbool_lambda():
    pass

MY_VALUE = 42 if bool_function else -1  # [missing-parentheses-for-call-in-test]
MY_2ND_VALUE = 42 if not bool_function() else -1
MY_THIRD_VALUE = 42 if bool_lambda else -1  # [missing-parentheses-for-call-in-test]
MY_FOURTH_VALUE = 42 if nonbool_lambda else -1  # [missing-parentheses-for-call-in-test]

[x for x in range(100) if bool_function]  # [missing-parentheses-for-call-in-test]
[x for x in range(100) if bool_lambda]  # [missing-parentheses-for-call-in-test]
[x for x in range(100) if not bool_function()]
[x for x in range(100) if not bool_lambda()]
[x for x in range(100) if nonbool_lambda]  # [missing-parentheses-for-call-in-test]
[x for x in range(100) if nonbool_function]  # [missing-parentheses-for-call-in-test]


def non_const_node_function():
    return (1, 2, 42)


if non_const_node_function:  # [missing-parentheses-for-call-in-test]
    pass


def yielding_function():
    yield 42


if yielding_function:  # [missing-parentheses-for-call-in-test]
    pass

if not yielding_function():
    pass


def is_it_a_good_day():
    """Seems like this is not working in python 3.11 ?"""
    return random.choice([True, False])


if is_it_a_good_day:  # [missing-parentheses-for-call-in-test]
    print("Today is a good day!")
elif yielding_function:  # [missing-parentheses-for-call-in-test]
    print("multiple condition")


def _cpu_count() -> int:
    """Reproducer for an inference error that can happen in
    missing-parentheses-for-call-in-test"""
    sched_getaffinity = getattr(os, "sched_getaffinity", None)
    if sched_getaffinity:
        print(sched_getaffinity)
    elif multiprocessing:
        multiprocessing.cpu_count()



# pylint: disable=missing-docstring, superfluous-parens


try:
    1/0
except (ValueError | TypeError): # [catching-non-exception,wrong-exception-operation]
    pass

try:
    1/0
except (ValueError + TypeError): # [wrong-exception-operation]
    pass


try:
    1/0
except (ValueError < TypeError): # [wrong-exception-operation]
    pass


# Concatenation of exception type tuples
DIVISION_BY_ZERO = (ZeroDivisionError,)
VALUE_ERROR = (ValueError,)
UNINFERABLE = DIVISION_BY_ZERO | VALUE_ERROR

try:
    1/0
except (ValueError, ) + DIVISION_BY_ZERO:
    pass

try:
    1/0
except (ValueError, ) | DIVISION_BY_ZERO:  # [wrong-exception-operation]
    pass

try:
    1/0
except (ValueError, ) + UNINFERABLE:
    pass



# pylint: disable=missing-docstring,too-few-public-methods

def test_unused(first, second, _not_used): # [unused-argument, unused-argument]
    pass


def test_prefixed_with_ignored(first, ignored_second):
    first()


def test_prefixed_with_unused(first, unused_second):
    first()

# for Sub.inherited, only the warning for "aay" is desired.
# The warnings for "aab" and "aac"  are most likely false positives though,
# because there could be another subclass that overrides the same method and does
# use the arguments (e.g. Sub2)


class Base:
    "parent"
    def inherited(self, aaa, aab, aac):
        "abstract method"
        raise NotImplementedError

class Sub(Base):
    "child 1"
    def inherited(self, aaa, aab, aac):
        "overridden method, though don't use every argument"
        return aaa

    def newmethod(self, aax, aay):  # [unused-argument]
        "another method, warning for aay desired"
        return self, aax

class Sub2(Base):
    "child 1"

    def inherited(self, aaa, aab, aac):
        "overridden method, use every argument"
        return aaa + aab + aac

def metadata_from_dict(key):
    """
    Should not raise unused-argument message because key is
    used inside comprehension dict
    """
    return {key: str(value) for key, value in key.items()}


def metadata_from_dict_2(key):
    """Similar, but with more nesting"""
    return {key: (a, b) for key, (a, b) in key.items()}


# pylint: disable=too-few-public-methods, wrong-import-position


def function(arg=1):  # [unused-argument]
    """ignore arg"""


class AAAA:
    """dummy class"""

    def method(self, arg):  # [unused-argument]
        """dummy method"""
        print(self)
    def __init__(self, *unused_args, **unused_kwargs):
        pass

    @classmethod
    def selected(cls, *args, **kwargs):  # [unused-argument, unused-argument]
        """called by the registry when the vobject has been selected.
        """
        return cls

    def using_inner_function(self, etype, size=1):
        """return a fake result set for a particular entity type"""
        rset = AAAA([('A',)]*size, f'{etype} X',
                    description=[(etype,)]*size)
        def inner(row, col=0, etype=etype, req=self, rset=rset):
            """inner using all its argument"""
            # pylint: disable=maybe-no-member
            return req.vreg.etype_class(etype)(req, rset, row, col)
        # pylint: disable = attribute-defined-outside-init
        rset.get_entity = inner

class BBBB:
    """dummy class"""

    def __init__(self, arg):  # [unused-argument]
        """Constructor with an extra parameter. Should raise a warning"""
        self.spam = 1


# Regression test for https://github.com/pylint-dev/pylint/issues/5771
# involving keyword-only arguments
class Ancestor:
    def __init__(self):
        self.thing = None

    def set_thing(self, thing, *, other=None):  # [unused-argument]
        self.thing = thing

class Descendant(Ancestor):
    def set_thing(self, thing, *, other=None):
        """Subclass does not raise unused-argument"""
        self.thing = thing


# Test that Class with both `__init__` and `__new__` don't check
# on `__new__` for unused arguments

# pylint: disable=invalid-name

class TestClassWithInitAndNew:
    def __init__(self, argA, argB):
        self.argA = argA
        self.argB = argB

    def __new__(cls, argA, argB):
        return object.__new__(cls)

# Test that `__new__` method is checked for unused arguments
# when `__init__` is not in the Class

class TestClassWithOnlyNew:
    def __new__(cls, argA, argB): # [unused-argument, unused-argument]
        return object.__new__(cls)



# pylint: disable=missing-docstring,too-few-public-methods

def baz():  # [disallowed-name]
    pass

class foo():  # [disallowed-name]
    pass

foo = {}.keys()  # [disallowed-name]
foo = 42  # [disallowed-name]
aaa = 42  # [invalid-name]



"""
Test that a constant parameter of `typing.Annotated` does not emit `unused-import`.
`typing.Annotated` was introduced in Python version 3.9
"""

from pathlib import Path  # [unused-import]
import typing as t


example: t.Annotated[str, "Path"] = "/foo/bar"



"""
https://github.com/pylint-dev/pylint/issues/4895
"""

# pylint: disable=missing-docstring

# Disabled because of a bug with pypy 3.8 see
# https://github.com/pylint-dev/pylint/pull/7918#issuecomment-1352737369
# pylint: disable=multiple-statements

from __future__ import annotations

from collections.abc import Callable
from dataclasses import dataclass


@dataclass
class Metric:
    function: Callable[..., tuple[int, int]]

    def update(self):
        _, _ = self.function()



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

"""Non regression tests for pylint, which requires a too specific configuration
to be incorporated in the automatic functional test framework.
"""

# pylint: disable=redefined-outer-name

from __future__ import annotations

import os
import sys
from collections.abc import Callable, Iterator
from os.path import abspath, dirname, join
from typing import cast

import pytest

from pylint import testutils
from pylint.lint.pylinter import PyLinter
from pylint.utils.linterstats import LinterStats

REGR_DATA = join(dirname(abspath(__file__)), "regrtest_data")
sys.path.insert(1, REGR_DATA)


@pytest.fixture(scope="module")
def reporter() -> type[testutils.GenericTestReporter]:
    return testutils.GenericTestReporter


@pytest.fixture(scope="module")
def disable() -> list[str]:
    return ["I"]


@pytest.fixture
def finalize_linter(linter: PyLinter) -> Iterator[PyLinter]:
    """Call reporter.finalize() to clean up pending messages if a test
    finished badly.
    """
    yield linter
    linter.reporter = cast(  # Due to fixture
        testutils.GenericTestReporter, linter.reporter
    )
    linter.reporter.finalize()


def Equals(expected: str) -> Callable[[str], bool]:
    return lambda got: got == expected


@pytest.mark.parametrize(
    "file_names, check",
    [
        (["package.__init__"], Equals("")),
        (["precedence_test"], Equals("")),
        (["import_package_subpackage_module"], Equals("")),
        (["pylint.checkers.__init__"], lambda x: "__path__" not in x),
        ([join(REGR_DATA, "classdoc_usage.py")], Equals("")),
        ([join(REGR_DATA, "module_global.py")], Equals("")),
        ([join(REGR_DATA, "decimal_inference.py")], Equals("")),
        ([join(REGR_DATA, "absimp", "string.py")], Equals("")),
        ([join(REGR_DATA, "bad_package")], lambda x: "Unused import missing" in x),
    ],
)
def test_package(
    finalize_linter: PyLinter, file_names: list[str], check: Callable[[str], bool]
) -> None:
    finalize_linter.check(file_names)
    finalize_linter.reporter = cast(  # Due to fixture
        testutils.GenericTestReporter, finalize_linter.reporter
    )
    got = finalize_linter.reporter.finalize().strip()
    assert check(got)


@pytest.mark.parametrize(
    "file_names",
    [
        [join(REGR_DATA, "import_assign.py")],
        [join(REGR_DATA, "special_attr_scope_lookup_crash.py")],
        [join(REGR_DATA, "try_finally_disable_msg_crash")],
    ],
)
def test_crash(finalize_linter: PyLinter, file_names: list[str]) -> None:
    finalize_linter.check(file_names)


@pytest.mark.parametrize(
    "fname", [x for x in os.listdir(REGR_DATA) if x.endswith("_crash.py")]
)
def test_descriptor_crash(fname: str, finalize_linter: PyLinter) -> None:
    finalize_linter.check([join(REGR_DATA, fname)])
    finalize_linter.reporter = cast(  # Due to fixture
        testutils.GenericTestReporter, finalize_linter.reporter
    )
    finalize_linter.reporter.finalize().strip()


@pytest.fixture
def modify_path() -> Iterator[None]:
    cwd = os.getcwd()
    sys.path.insert(0, "")
    yield
    sys.path.pop(0)
    os.chdir(cwd)


@pytest.mark.usefixtures("modify_path")
def test_check_package___init__(finalize_linter: PyLinter) -> None:
    filename = ["package.__init__"]
    finalize_linter.check(filename)
    checked = list(finalize_linter.stats.by_module.keys())
    assert sorted(checked) == sorted(filename)

    os.chdir(join(REGR_DATA, "package"))
    finalize_linter.stats = LinterStats()
    finalize_linter.check(["__init__"])
    checked = list(finalize_linter.stats.by_module.keys())
    assert checked == ["__init__"]


@pytest.mark.timeout(30)
@pytest.mark.parametrize("file_names", ([join(REGR_DATA, "hang", "pkg4972.string")],))
def test_hang(finalize_linter: PyLinter, file_names: list[str]) -> None:
    finalize_linter.check(file_names)



"""Tests for the use of typing.final whenever the py-version is set < 3.8"""
# pylint: disable=missing-class-docstring, no-member, too-few-public-methods, missing-function-docstring, no-name-in-module, reimported

import typing
import typing as mytyping
from typing import final
from typing import final as myfinal


@final  # [using-final-decorator-in-unsupported-version]
class MyClass1:
    @final  # [using-final-decorator-in-unsupported-version]
    @final  # [using-final-decorator-in-unsupported-version]
    def my_method(self):
        pass


@myfinal  # [using-final-decorator-in-unsupported-version]
class MyClass2:
    @myfinal  # [using-final-decorator-in-unsupported-version]
    def my_method(self):
        pass


@typing.final  # [using-final-decorator-in-unsupported-version]
class MyClass3:
    @typing.final  # [using-final-decorator-in-unsupported-version]
    def my_method(self):
        pass


@mytyping.final  # [using-final-decorator-in-unsupported-version]
class MyClass4:
    @mytyping.final  # [using-final-decorator-in-unsupported-version]
    def my_method(self):
        pass



# ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789  # pylint: disable=line-too-long
# See https://github.com/pylint-dev/pylint/issues/3368



"""Test that invalid module name imports causes correct error"""
# pylint: disable=import-error, wrong-import-position, unused-wildcard-import, wildcard-import, wrong-import-order
import sys
import os

# allow module imports to test that this is indeed a valid python file
SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.dirname(SCRIPT_DIR))

import non_ascii_name_loł as ok
from pathlib import *   # test that star imports work correctly and give no error

print(ok)



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

from __future__ import annotations

import codecs
import io
import shutil
from pathlib import Path
from typing import cast

import pytest
from astroid import nodes

import pylint.checkers.unicode
import pylint.interfaces
import pylint.testutils

from . import CODEC_AND_MSG, FakeNode

UNICODE_TESTS = Path(__file__).parent.parent.parent / "regrtest_data" / "unicode"


class TestInvalidEncoding(pylint.testutils.CheckerTestCase):
    CHECKER_CLASS = pylint.checkers.unicode.UnicodeChecker
    checker: pylint.checkers.unicode.UnicodeChecker

    @pytest.mark.parametrize(
        "test_file, line_no",
        [
            pytest.param(
                "pep_bidirectional_utf_16_le_no_bom.txt",
                2,
                marks=pytest.mark.xfail(
                    reason="Python currently doesn't support UTF-16 code detection"
                ),
            ),
            pytest.param(
                "pep_bidirectional_utf_32_le_no_bom.txt",
                2,
                marks=pytest.mark.xfail(
                    reason="Python currently doesn't support UTF-32 code detection"
                ),
            ),
            # A note to the xfails above: If you open these files in an editor, you
            # only will see garbage if you don't select the correct encoding by hand.
            # Still maybe in the future the python way of defining the encoding could
            # work - even so it is unlikely as the first line is not ASCII and would
            # have to be treated differently anyway...
            ("pep_bidirectional_utf_16_bom.txt", 1),
            ("pep_bidirectional_utf_32_bom.txt", 1),
        ],
    )
    def test_invalid_unicode_files(
        self, tmp_path: Path, test_file: str, line_no: int
    ) -> None:
        test_file_path = UNICODE_TESTS / test_file
        target = shutil.copy(
            test_file_path, tmp_path / test_file.replace(".txt", ".py")
        )

        # Fake node as otherwise we get syntax errors etc...
        # So currently the UTF-16/UTF-32 tests does not work, as UTF-16 / UTF-32
        # is not really working at all in in Python, but checking it now already
        # is future save in case that changes....

        module = FakeNode(Path(target).read_bytes())

        with self.assertAddsMessages(
            pylint.testutils.MessageTest(
                msg_id="invalid-unicode-codec",
                confidence=pylint.interfaces.HIGH,
                # node=module,
                line=line_no,
                end_line=1,
                col_offset=None,
                end_col_offset=None,
            ),
            pylint.testutils.MessageTest(
                msg_id="bidirectional-unicode",
                confidence=pylint.interfaces.HIGH,
                # node=module,
                line=line_no + 2,
                end_line=line_no + 2,
                col_offset=0,
                end_col_offset=37,
            ),
        ):
            self.checker.process_module(cast(nodes.Module, module))

    @pytest.mark.parametrize(
        "content, codec, line",
        [
            pytest.param(b"# Nothing", "utf-8", 1, id="default_utf8"),
            pytest.param(b"# coding: latin-1", "iso-8859-1", 1, id="pep263_latin1"),
            pytest.param(
                b"#!/usr/bin/python\n# coding: latin-1",
                "iso-8859-1",
                2,
                id="pep263_latin1_multiline",
            ),
            pytest.param(b"# coding: ascii", "ascii", 1, id="pep263_ascii"),
            pytest.param(b"# coding: UTF-8", "utf-8", 1, id="pep263_utf-8"),
            # This looks correct but is actually wrong. If you would try to decode
            # the byte to utf-16be it would fail
            pytest.param(
                b"# coding: UTF-16le", "utf-16le", 1, id="pep263_utf-16le_fake"
            ),
            # This contains no bom but a correct encoding line in none ascii
            # So this fails at the moment
            pytest.param(
                "# coding: UTF-16le".encode("utf-16le"),
                "utf-16le",
                1,
                id="pep263_utf-16le_real",
                marks=pytest.mark.xfail(reason="Currently not supported by Python"),
            ),
            *(
                pytest.param(bom, codec, 1, id=f"bom_{codec}")
                for codec, bom in (
                    ("utf-8", codecs.BOM_UTF8),
                    ("utf-16le", codecs.BOM_UTF16_LE),
                    ("utf-16be", codecs.BOM_UTF16_BE),
                    ("utf-32le", codecs.BOM_UTF32_LE),
                    ("utf-32be", codecs.BOM_UTF32_BE),
                )
            ),
        ],
    )
    def test__determine_codec(self, content: bytes, codec: str, line: int) -> None:
        """The codec determined should be exact no matter what we throw at it."""
        assert self.checker._determine_codec(io.BytesIO(content)) == (codec, line)

    def test__determine_codec_raises_syntax_error_on_invalid_input(self) -> None:
        """Invalid input should lead to a SyntaxError."""
        with pytest.raises(SyntaxError):
            self.checker._determine_codec(io.BytesIO(b"\x80abc"))

    @pytest.mark.parametrize(
        "codec, msg",
        (pytest.param(codec, msg, id=codec) for codec, msg in CODEC_AND_MSG),
    )
    def test___check_codec(
        self, codec: str, msg: tuple[pylint.testutils.MessageTest]
    ) -> None:
        with self.assertAddsMessages(*msg):
            self.checker._check_codec(codec, 1)



"""Profiles basic -jX functionality."""

# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

# pylint: disable=missing-function-docstring

from __future__ import annotations

import os
import pprint
from pathlib import Path

import pytest
from git.repo import Repo

from pylint.testutils import GenericTestReporter as Reporter
from pylint.testutils._run import _Run as Run


def _get_py_files(scanpath: str) -> list[str]:
    assert os.path.exists(scanpath), f"Dir not found {scanpath}"

    filepaths: list[str] = []
    for dirpath, dirnames, filenames in os.walk(scanpath):
        dirnames[:] = [dirname for dirname in dirnames if dirname != "__pycache__"]
        filepaths.extend(
            [
                os.path.join(dirpath, filename)
                for filename in filenames
                if filename.endswith(".py")
            ]
        )
    return filepaths


@pytest.mark.skipif(
    not os.environ.get("PYTEST_PROFILE_EXTERNAL", False),
    reason="PYTEST_PROFILE_EXTERNAL, not set, assuming not a profile run",
)
@pytest.mark.parametrize(
    "name,git_repo", [("numpy", "https://github.com/numpy/numpy.git")]
)
def test_run(tmp_path: Path, name: str, git_repo: str) -> None:
    """Runs pylint against external sources."""
    checkoutdir = tmp_path / name
    checkoutdir.mkdir()
    Repo.clone_from(url=git_repo, to_path=checkoutdir, depth=1)
    filepaths = _get_py_files(scanpath=str(checkoutdir))
    print(f"Have {len(filepaths)} files")

    runner = Run(filepaths, reporter=Reporter(), exit=False)

    print(
        f"Had {len(filepaths)} files with {len(runner.linter.reporter.messages)} messages"
    )
    pprint.pprint(runner.linter.reporter.messages)



from typing import final


@final
class PlatypusData:
    """General Platypus data."""

    average_length = 46
    average_body_temperature = 32


class FluorescentPlaytipus(PlatypusData):  # [subclassed-final-class]
    """Playtipus with fluorescent fur."""



class cat:  # [invalid-name]
    def Meow(self, NUMBER_OF_MEOW):  # [invalid-name, invalid-name]
        print("Meow" * NUMBER_OF_MEOW)
        return NUMBER_OF_MEOW


Cat = cat().Meow(42)  # [invalid-name]



"""Suspicious str.strip calls."""
# pylint: disable=redundant-u-string-prefix


''.strip('yo')
''.strip()

u''.strip('http://')  # [bad-str-strip-call]
u''.lstrip('http://')  # [bad-str-strip-call]
b''.rstrip('http://')  # [bad-str-strip-call]



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

from __future__ import annotations

from collections.abc import Callable

import pytest
from astroid.nodes.scoped_nodes import Module

from pylint.lint import augmented_sys_path, discover_package_path
from pylint.pyreverse.inspector import Project, project_from_files
from pylint.testutils.pyreverse import PyreverseConfig
from pylint.typing import GetProjectCallable


@pytest.fixture()
def default_config() -> PyreverseConfig:
    return PyreverseConfig()


@pytest.fixture()
def colorized_dot_config() -> PyreverseConfig:
    return PyreverseConfig(
        output_format="dot",
        colorized=True,
    )


@pytest.fixture()
def no_standalone_dot_config() -> PyreverseConfig:
    return PyreverseConfig(
        output_format="dot",
        no_standalone=True,
    )


@pytest.fixture()
def puml_config() -> PyreverseConfig:
    return PyreverseConfig(
        output_format="puml",
    )


@pytest.fixture()
def colorized_puml_config() -> PyreverseConfig:
    return PyreverseConfig(
        output_format="puml",
        colorized=True,
    )


@pytest.fixture()
def mmd_config() -> PyreverseConfig:
    return PyreverseConfig(
        output_format="mmd",
        colorized=False,
    )


@pytest.fixture()
def html_config() -> PyreverseConfig:
    return PyreverseConfig(
        output_format="html",
        colorized=False,
    )


@pytest.fixture(scope="session")
def get_project() -> GetProjectCallable:
    def _get_project(module: str, name: str | None = "No Name") -> Project:
        """Return an astroid project representation."""

        def _astroid_wrapper(
            func: Callable[[str], Module], modname: str, _verbose: bool = False
        ) -> Module:
            return func(modname)

        with augmented_sys_path([discover_package_path(module, [])]):
            project = project_from_files([module], _astroid_wrapper, project_name=name)
        return project

    return _get_project



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

"""Functional/non regression tests for pylint."""

from __future__ import annotations

import re
import sys
from os.path import abspath, dirname, join

import pytest

from pylint.testutils import UPDATE_FILE, UPDATE_OPTION, _get_tests_info, linter
from pylint.testutils.reporter_for_tests import GenericTestReporter
from pylint.testutils.utils import _test_cwd

TESTS_DIR = dirname(abspath(__file__))
INPUT_DIR = join(TESTS_DIR, "input")
MSG_DIR = join(TESTS_DIR, "messages")


FILTER_RGX = None
INFO_TEST_RGX = re.compile(r"^func_i\d\d\d\d$")


def exception_str(
    self: Exception, ex: Exception  # pylint: disable=unused-argument
) -> str:
    """Function used to replace default __str__ method of exception instances
    This function is not typed because it is legacy code.
    """
    return f"in {ex.file}\n:: {', '.join(ex.args)}"  # type: ignore[attr-defined] # Defined in the caller


class LintTestUsingModule:
    INPUT_DIR: str | None = None
    DEFAULT_PACKAGE = "input"
    package = DEFAULT_PACKAGE
    linter = linter
    module: str | None = None
    depends: list[tuple[str, str]] | None = None
    output: str | None = None

    def _test_functionality(self) -> None:
        tocheck = [self.package + "." + self.module] if self.module else []
        if self.depends:
            tocheck += [
                self.package + f".{name.replace('.py', '')}" for name, _ in self.depends
            ]
        # given that TESTS_DIR could be treated as a namespace package
        # when under the current directory, cd to it so that "tests." is not
        # prepended to module names in the output of cyclic-import
        with _test_cwd(TESTS_DIR):
            self._test(tocheck)

    def _check_result(self, got: str) -> None:
        error_msg = (
            f"Wrong output for '{self.output}':\n"
            "You can update the expected output automatically with: '"
            f"python tests/test_func.py {UPDATE_OPTION}'\n\n"
        )
        assert self._get_expected() == got, error_msg

    def _test(self, tocheck: list[str]) -> None:
        if self.module and INFO_TEST_RGX.match(self.module):
            self.linter.enable("I")
        else:
            self.linter.disable("I")
        try:
            self.linter.check(tocheck)
        except Exception as ex:
            print(f"Exception: {ex} in {tocheck}:: {', '.join(ex.args)}")
            # This is legacy code we're trying to remove, not worth it to type correctly
            ex.file = tocheck  # type: ignore[attr-defined]
            print(ex)
            # This is legacy code we're trying to remove, not worth it to type correctly
            ex.__str__ = exception_str  # type: ignore[assignment]
            raise
        assert isinstance(self.linter.reporter, GenericTestReporter)
        self._check_result(self.linter.reporter.finalize())

    def _has_output(self) -> bool:
        return isinstance(self.module, str) and not self.module.startswith(
            "func_noerror_"
        )

    def _get_expected(self) -> str:
        if self._has_output() and self.output:
            with open(self.output, encoding="utf-8") as fobj:
                return fobj.read().strip() + "\n"
        else:
            return ""


class LintTestUpdate(LintTestUsingModule):
    def _check_result(self, got: str) -> None:
        if not self._has_output():
            return
        try:
            expected = self._get_expected()
        except OSError:
            expected = ""
        if got != expected:
            with open(self.output or "", "w", encoding="utf-8") as f:
                f.write(got)


def gen_tests(
    filter_rgx: str | re.Pattern[str] | None,
) -> list[tuple[str, str, list[tuple[str, str]]]]:
    if filter_rgx:
        is_to_run = re.compile(filter_rgx).search
    else:
        is_to_run = (  # noqa: E731, We're going to throw all this anyway
            lambda x: 1  # type: ignore[assignment] # pylint: disable=unnecessary-lambda-assignment
        )
    tests: list[tuple[str, str, list[tuple[str, str]]]] = []
    for module_file, messages_file in _get_tests_info(INPUT_DIR, MSG_DIR, "func_", ""):
        if not is_to_run(module_file) or module_file.endswith((".pyc", "$py.class")):
            continue
        base = module_file.replace(".py", "").split("_")[1]
        dependencies = _get_tests_info(INPUT_DIR, MSG_DIR, base, ".py")
        tests.append((module_file, messages_file, dependencies))
    if UPDATE_FILE.exists():
        return tests
    assert len(tests) < 13, "Please do not add new test cases here." + "\n".join(
        str(k) for k in tests if not k[2]
    )
    return tests


TEST_WITH_EXPECTED_DEPRECATION = ["func_excess_escapes.py"]


@pytest.mark.parametrize(
    "module_file,messages_file,dependencies",
    gen_tests(FILTER_RGX),
    ids=[o[0] for o in gen_tests(FILTER_RGX)],
)
def test_functionality(
    module_file: str,
    messages_file: str,
    dependencies: list[tuple[str, str]],
    recwarn: pytest.WarningsRecorder,
) -> None:
    __test_functionality(module_file, messages_file, dependencies)
    if recwarn.list:
        if module_file in TEST_WITH_EXPECTED_DEPRECATION and sys.version_info.minor > 5:
            assert any(
                "invalid escape sequence" in str(i.message)
                for i in recwarn.list
                if issubclass(i.category, DeprecationWarning)
            )


def __test_functionality(
    module_file: str, messages_file: str, dependencies: list[tuple[str, str]]
) -> None:
    lint_test = LintTestUpdate() if UPDATE_FILE.exists() else LintTestUsingModule()
    lint_test.module = module_file.replace(".py", "")
    lint_test.output = messages_file
    lint_test.depends = dependencies or None
    lint_test.INPUT_DIR = INPUT_DIR
    lint_test._test_functionality()


if __name__ == "__main__":
    if UPDATE_OPTION in sys.argv:
        UPDATE_FILE.touch()
        sys.argv.remove(UPDATE_OPTION)
    if len(sys.argv) > 1:
        FILTER_RGX = sys.argv[1]
        del sys.argv[1]
    try:
        pytest.main(sys.argv)
    finally:
        if UPDATE_FILE.exists():
            UPDATE_FILE.unlink()



"""Non ASCII name in class variable"""


class OkayIsh:
    """Class docstring"""

    def public(self):
        """Be public!"""
        print(self)

    def __init__(self):
        self.łoopback = "invalid"  # [non-ascii-name]

    def foobar(self):
        """do something"""
        # Usage should not raise a second error
        return self.łoopback

def main():
    """main function"""
    # Usage should not raise a second error
    barrrr = OkayIsh()
    barrrr.foobar()
    test = barrrr.łoopback
    print(test)



# pylint: disable=missing-docstring,unused-import
# https://github.com/pylint-dev/pylint/issues/3595

import collections

class Example:
    def __init__(self):
        self.prev()
        self.next()
    def prev(self):
        pass
    def next(self):
        pass



"""Tests for the unused-variable message in assignment expressions"""


def typed_assignment_in_function_default( # [unused-variable]
    param: str = (typed_default := "walrus"),  # [unused-variable]
) -> None:
    """An unused annotated assignment expression in a default parameter should emit"""
    return param


def assignment_in_function_default( # [unused-variable]
    param=(default := "walrus"),  # [unused-variable]
) -> None:
    """An unused assignment expression in a default parameter should emit"""
    return param


def assignment_used_in_function_scope( # [unused-variable]
    param=(function_default := "walrus"),
) -> None:
    """An used assignment expression in a default parameter should not emit"""
    print(function_default)
    return param


def assignment_used_in_global_scope( # [unused-variable]
    param=(global_default := "walrus"),
) -> None:
    """An used assignment expression in a default parameter should not emit"""
    return param

print(global_default)



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

"""Unit test for the diagrams modules."""

from __future__ import annotations

from pylint.pyreverse.diadefslib import DefaultDiadefGenerator, DiadefsHandler
from pylint.pyreverse.inspector import Linker
from pylint.testutils.pyreverse import PyreverseConfig
from pylint.typing import GetProjectCallable


def test_property_handling(
    default_config: PyreverseConfig, get_project: GetProjectCallable
) -> None:
    project = get_project("data.property_pattern")
    class_diagram = DefaultDiadefGenerator(
        Linker(project), DiadefsHandler(default_config)
    ).visit(project)[0]
    obj = class_diagram.classe("PropertyPatterns")
    assert len(class_diagram.get_methods(obj.node)) == 0
    assert class_diagram.get_attrs(obj.node) == ["prop1", "prop2"]



"""
Regression test for https://github.com/pylint-dev/pylint/issues/3976

E1123: Unexpected keyword argument 'include_extras' in function call (unexpected-keyword-arg)
"""

import typing_extensions


def function():
    """Simple function"""


typing_extensions.get_type_hints(function, include_extras=True)



from argparse import _AttributeHolder, _SubParsersAction  # [import-private-name]

attr_holder = _AttributeHolder()


def add_sub_parser(sub_parsers: _SubParsersAction):
    sub_parsers.add_parser("my_subparser")
    # ...



"""Test to see if an f-string would be possible and consider-using-f-string should be raised"""
# pylint: disable=unused-variable, invalid-name, missing-function-docstring, pointless-statement
# pylint: disable=expression-not-assigned, repeated-keyword, too-many-locals

PARAM_1 = PARAM_2 = PARAM_3 = 1
PARAM_LIST = [PARAM_1, PARAM_2, PARAM_3]
PARAM_LIST_SINGLE = [PARAM_1]
PARAM_DICT = {"Param_1": PARAM_1, "Param_2": PARAM_2, "Param_3": PARAM_3}
PARAM_DICT_SINGLE = {"Param_1": PARAM_1}


def return_parameter():
    return PARAM_1


def return_list():
    return PARAM_LIST


def return_dict():
    return PARAM_DICT


def print_good():
    print("String {}, {} or {}".format(*PARAM_LIST))
    print("String {}, {}, {} or {}".format(*PARAM_LIST_SINGLE, *PARAM_LIST))
    print("String {Param}, {}, {} or {}".format(Param=PARAM_1, *PARAM_LIST))
    print("String {Param} {Param}".format(Param=PARAM_1))
    print("{Param_1} {Param_2}".format(**PARAM_DICT))
    print("{Param_1} {Param_2} {Param_3}".format(**PARAM_DICT_SINGLE, **PARAM_DICT))
    print("{Param_1} {Param_2} {Param_3}".format(Param_1=PARAM_1, **PARAM_DICT))
    print("{Param_1} {Param_2}".format(**PARAM_DICT))
    print("{Param_1} {Param_2}".format(**return_dict()))
    print("%(Param_1)s %(Param_2)s" % PARAM_LIST)
    print("%(Param_1)s %(Param_2)s" % PARAM_DICT)
    print("%(Param_1)s %(Param_2)s" % return_dict())
    print("{a[Param_1]}{a[Param_2]}".format(a=PARAM_DICT))
    print("{}".format("\n"))
    print("{}".format("\n".join(i for i in "string")))
    print("%s" % "\n")
    print("%s" % "\n".join(i for i in "string"))
    print("{%s}%s" % (PARAM_1, PARAM_2))


def print_bad():
    print("String %f" % PARAM_1)  # [consider-using-f-string]
    print("String {}".format(PARAM_1))  # [consider-using-f-string]
    print("String {Param_1}".format(Param_1=PARAM_1))  # [consider-using-f-string]
    print("{} {}".format(PARAM_1, PARAM_2))  # [consider-using-f-string]
    print("{Par_1}{Par_2}".format(Par_1=PARAM_1, Par_2=PARAM_2))  # [consider-using-f-string]
    print("{Param_1}".format(*PARAM_LIST_SINGLE))  # [consider-using-f-string]
    print("{Param_1}".format(**PARAM_DICT_SINGLE))  # [consider-using-f-string]
    print("String %s" % (PARAM_1))  # [consider-using-f-string]
    print("String %s %s" % (PARAM_1, PARAM_2))  # [consider-using-f-string]
    print("String %s" % (PARAM_LIST_SINGLE))  # [consider-using-f-string]


def statement_good():
    "String {}, {} or {}".format(*PARAM_LIST)
    "String {}, {}, {} or {}".format(*PARAM_LIST_SINGLE, *PARAM_LIST)
    "String {Param}, {}, {} or {}".format(Param=PARAM_1, *PARAM_LIST)
    "String {Param} {Param}".format(Param=PARAM_1)
    "{Param_1} {Param_2}".format(**PARAM_DICT)
    "{Param_1} {Param_2} {Param_3}".format(**PARAM_DICT_SINGLE, **PARAM_DICT)
    "{Param_1} {Param_2} {Param_3}".format(Param_1=PARAM_1, **PARAM_DICT)
    "{Param_1} {Param_2}".format(**PARAM_DICT)
    "{Param_1} {Param_2}".format(**return_dict())
    "%(Param_1)s %(Param_2)s" % PARAM_LIST
    "%(Param_1)s %(Param_2)s" % PARAM_DICT
    "%(Param_1)s %(Param_2)s" % return_dict()
    "{a[Param_1]}{a[Param_2]}".format(a=PARAM_DICT)
    "{}".format("\n")
    "{}".format("\n".join(i for i in "string"))
    "%s" % "\n"
    "%s" % "\n".join(i for i in "string")
    1 % "str"
    (1, 2) % 'garbage'


def statement_bad():
    "String %f" % PARAM_1  # [consider-using-f-string]
    "String {}".format(PARAM_1)  # [consider-using-f-string]
    "String {Param_1}".format(Param_1=PARAM_1)  # [consider-using-f-string]
    "{} {}".format(PARAM_1, PARAM_2)  # [consider-using-f-string]
    "{Par_1}{Par_2}".format(Par_1=PARAM_1, Par_2=PARAM_2)  # [consider-using-f-string]
    "{Param_1}".format(*PARAM_LIST_SINGLE)  # [consider-using-f-string]
    "{Param_1}".format(**PARAM_DICT_SINGLE)  # [consider-using-f-string]
    "String %s" % (PARAM_1)  # [consider-using-f-string]
    "String %s %s" % (PARAM_1, PARAM_2)  # [consider-using-f-string]
    "String %s" % (PARAM_LIST_SINGLE)  # [consider-using-f-string]


def assignment_good():
    A = "String {}, {} or {}".format(*PARAM_LIST)
    B = "String {}, {}, {} or {}".format(*PARAM_LIST_SINGLE, *PARAM_LIST)
    C = "String {Param}, {}, {} or {}".format(Param=PARAM_1, *PARAM_LIST)
    D = "String {Param} {Param}".format(Param=PARAM_1)
    E = "{Param_1} {Param_2}".format(**PARAM_DICT)
    F = "{Param_1} {Param_2} {Param_3}".format(**PARAM_DICT_SINGLE, **PARAM_DICT)
    G = "{Param_1} {Param_2} {Param_3}".format(Param_1=PARAM_1, **PARAM_DICT)
    H = "{Param_1} {Param_2}".format(**PARAM_DICT)
    I = "{Param_1} {Param_2}".format(**return_dict())
    J = "%(Param_1)s %(Param_2)s" % PARAM_LIST
    K = "%(Param_1)s %(Param_2)s" % PARAM_DICT
    L = "%(Param_1)s %(Param_2)s" % return_dict()
    M = "{a[Param_1]}{a[Param_2]}".format(a=PARAM_DICT)
    N = "{Param}".format
    O = "%s" % "\n"
    P = "%s" % "\n".join(i for i in "string")


def assignment_bad():
    a = "String %f" % PARAM_1  # [consider-using-f-string]
    b = "String {}".format(PARAM_1)  # [consider-using-f-string]
    c = "String {Param_1}".format(Param_1=PARAM_1)  # [consider-using-f-string]
    d = "{} {}".format(PARAM_1, PARAM_2)  # [consider-using-f-string]
    e = "{Par_1}{Par_2}".format(Par_1=PARAM_1, Par_2=PARAM_2)  # [consider-using-f-string]
    f = "{Param_1}".format(*PARAM_LIST_SINGLE)  # [consider-using-f-string]
    g = "{Param_1}".format(**PARAM_DICT_SINGLE)  # [consider-using-f-string]
    h = "String %s" % (PARAM_1)  # [consider-using-f-string]
    i = "String %s %s" % (PARAM_1, PARAM_2)  # [consider-using-f-string]
    j = "String %s" % (PARAM_LIST_SINGLE)  # [consider-using-f-string]


def regression_tests():
    # Referencing .format in a kwarg should not be warned
    def wrap_print(value):
        print(value)

    wrap_print(value="{}".format)



"""Tests for missing-yield-doc and missing-yield-type-doc with accept-no-yields-doc = no"""
# pylint: disable=missing-function-docstring, unused-argument, function-redefined

from typing import Iterator


# Test missing docstring
def my_func(self):  # [missing-yield-doc, missing-yield-type-doc]
    yield False


# This function doesn't require a docstring, because its name starts
# with an '_' (no-docstring-rgx):
def _function(some_arg: int) -> Iterator[int]:
    yield some_arg



"""invalid ASCII char in a function definition"""
# pylint: disable=invalid-name


def sayHello():
    """Greetings"""
    print("Hello, World!")


# +3: [non-ascii-name]


def sayНello():
    """From Russia with Love"""
    print("Goodbye, World!")


# Usage should not raise a second error
sayНello()



"""Test for the regression on (outer)-class traversal for private methods
MyClass does not have an outerclass which previously crashed the protected-access check
"""
# pylint: disable=too-few-public-methods


class MyClass:
    """Test class"""

    @staticmethod
    def _a_private_method():
        """Private method that references the class itself"""
        return MySecondClass.MyClass._a_private_method()  # [protected-access]


class MySecondClass:
    """Class that uses MyClass"""

    MyClass = MyClass



"""Tests for missing-param-doc and missing-type-doc for non-specified style docstrings
with accept-no-param-doc = no and docstring-min-length = 3
"""
# pylint: disable=invalid-name, unused-argument

# Example of a function that is less than 'docstring-min-length' config option
# No error message is emitted.
def test_skip_docstring_min_length(x, y):
    """function is too short and is missing parameter documentation"""



"""Tests for used-before-assignment with assignments in except handlers after
try blocks with return statements.
See: https://github.com/pylint-dev/pylint/issues/5500.
"""
# pylint: disable=inconsistent-return-statements


# Named expressions
def func_ok_namedexpr_1(var):
    """'msg' is defined in one handler with a named expression under an if."""
    try:
        return 1 / var.some_other_func()
    except AttributeError:
        if (msg := var.get_msg()):
            pass
    except ZeroDivisionError:
        msg = "Division by 0"
    print(msg)


def func_ok_namedexpr_2(var):
    """'msg' is defined in one handler with a named expression occurring
    in a call used in an if test.
    """
    try:
        return 1 / var.some_other_func()
    except AttributeError:
        if print(msg := var.get_msg()):
            pass
    except ZeroDivisionError:
        msg = "Division by 0"
    print(msg)


def func_ok_namedexpr_3(var):
    """'msg' is defined in one handler with a named expression occurring
    as a keyword in a call used in an if test.
    """
    try:
        return 1 / var.some_other_func()
    except AttributeError:
        if print("zero!", "here", sep=(msg := var.get_sep())):
            pass
    except ZeroDivisionError:
        msg = "Division by 0"
    print(msg)



"""Tests for missing-return-doc and missing-return-type-doc"""
# pylint: disable=function-redefined, invalid-name, undefined-variable, missing-function-docstring
# pylint: disable=unused-argument


def ignores_no_docstring(self):
    return False


def ignores_unknown_style(self):
    """This is a docstring."""
    return False



# pylint: disable=trailing-whitespace, missing-module-docstring, trailing-newlines, missing-final-newline

# +2: [invalid-character-sub]
SUB = '''
--><---
'''  # bad char  sub should be only used as \x1A



# The name is invalid, but we should not analyse this file
# Because its filename resembles an Emacs file lock ignored by default
# https://www.gnu.org/software/emacs/manual/html_node/elisp/File-Locks.html
# See https://github.com/pylint-dev/pylint/issues/367



""" Checks that classes uses valid __slots__ """

# pylint: disable=too-few-public-methods, missing-docstring
# pylint: disable=using-constant-test, wrong-import-position, no-else-return, line-too-long, unused-private-member
from collections import deque

def func():
    if True:
        return ("a", "b", "c")
    else:
        return [str(var) for var in range(3)]


class NotIterable:
    def __iter_(self):
        """ do nothing """

class Good:
    __slots__ = ()

class SecondGood:
    __slots__ = []

class ThirdGood:
    __slots__ = ['a']

class FourthGood:
    __slots__ = (f'a{i}' for i in range(10))

class FifthGood:
    __slots__ = deque(["a", "b", "c"])

class SixthGood:
    __slots__ = {"a": "b", "c": "d"}

class SeventhGood:
    """type-annotated __slots__ with no value"""
    __slots__: str

class EigthGood:
    """Multiple __slots__ declared in the class"""
    x = 1
    if x:
        __slots__: str
    else:
        __slots__ = ("y",)

class Bad: # [invalid-slots]
    __slots__ = list

class SecondBad:  # [invalid-slots]
    __slots__ = 1

class ThirdBad:
    __slots__ = ('a', 2)  # [invalid-slots-object]

class FourthBad:  # [invalid-slots]
    __slots__ = NotIterable()

class FifthBad:
    __slots__ = ("a", "b", "")  # [invalid-slots-object]

class SixthBad:  # [single-string-used-for-slots]
    __slots__ = "a"

class SeventhBad:  # [single-string-used-for-slots]
    __slots__ = ('foo')  # [superfluous-parens]

class EighthBad:  # [single-string-used-for-slots]
    __slots__ = deque.__name__

class NinthBad:
    __slots__ = [str]  # [invalid-slots-object]

class TenthBad:
    __slots__ = [1 + 2 + 3]  # [invalid-slots-object]

class EleventhBad:  # [invalid-slots]
    __slots__ = None

class TwelfthBad:  # [invalid-slots]
    """One valid & one invalid __slots__ value"""
    x = 1
    if x:
        __slots__ = ("y",)
    else:
        __slots__ = None

class PotentiallyGood:
    __slots__ = func()

class PotentiallySecondGood:
    __slots__ = ('a', deque.__name__)


class Metaclass(type):

    def __iter__(cls):
        for value in range(10):
            yield str(value)


class IterableClass(metaclass=Metaclass):
    pass

class PotentiallyThirdGood:
    __slots__ = IterableClass

class PotentiallyFourthGood:
    __slots__ = Good.__slots__


class ValueInSlotConflict:
    __slots__ = ('first', 'second', 'third', 'fourth') # [class-variable-slots-conflict, class-variable-slots-conflict, class-variable-slots-conflict]
    first = None

    @property
    def third(self):
        return 42

    def fourth(self):
        return self.third


class Parent:
    first = 42


class ChildNotAffectedByValueInSlot(Parent):
    __slots__ = ('first', )



# Test for https://github.com/pylint-dev/pylint/issues/8189
class DuplicateFields():
    example1: int
    example2: int

    def __init__(self):
        self.example1 = 1
        self.example2 = 2


# Test for https://github.com/pylint-dev/pylint/issues/8522
class A:
    pass

class DuplicateArrows:
    a: A

    def __init__(self):
        self.a = A()



# Test for https://github.com/pylint-dev/pylint/issues/8888
class DuplicateAnnotations:
    def __init__(self) -> None:
        self.val: str | int = "1"
        self.lav: list[str] = []

    def bar(self) -> None:
        self.val = "2"
        self.lav = []



class Point:  # [multiple-constructor-doc]
    """Represents a point in the xy-coordinate plane.

    :param x: coordinate
    :param y: coordinate
    """

    def __init__(self, x, y):
        """Represents a point in the xy-coordinate plane.

        :param x: coordinate
        :param y: coordinate
        """
        self.x = x
        self.y = y



# pylint: disable=literal-comparison,missing-docstring, singleton-comparison

X = 123
Y = len('test')

if X is 0:  # [use-implicit-booleaness-not-comparison-to-zero]
    pass

if X is False:
    pass

if Y is not 0:  # [use-implicit-booleaness-not-comparison-to-zero]
    pass

if Y is not False:
    pass

if X == 0:  # [use-implicit-booleaness-not-comparison-to-zero]
    pass

if X == False:
    pass

if 0 == Y:  # [use-implicit-booleaness-not-comparison-to-zero]
    pass

if Y != 0:  # [use-implicit-booleaness-not-comparison-to-zero]
    pass

if 0 != X:  # [use-implicit-booleaness-not-comparison-to-zero]
    pass

if Y != False:
    pass

if X > 0:
    pass

if X < 0:
    pass

if 0 < X:
    pass

if 0 > X:
    pass



def num_to_word(x):  # [too-many-branches]
    if x == 0:
        return "zero"
    elif x == 1:
        return "one"
    elif x == 2:
        return "two"
    elif x == 3:
        return "three"
    elif x == 4:
        return "four"
    elif x == 5:
        return "five"
    elif x == 6:
        return "six"
    elif x == 7:
        return "seven"
    elif x == 8:
        return "eight"
    elif x == 9:
        return "nine"
    else:
        return None



import contextlib


@contextlib.contextmanager
def good_cm_except():
    contextvar = "acquired context"
    print("good cm enter")
    try:
        yield contextvar
    except GeneratorExit:
        print("good cm exit")


def genfunc_with_cm():
    with good_cm_except() as context:
        yield context * 2


def genfunc_with_discard():
    with good_cm_except():
        yield "discarded"


@contextlib.contextmanager
def good_cm_yield_none():
    print("good cm enter")
    yield
    print("good cm exit")


def genfunc_with_none_yield():
    with good_cm_yield_none() as var:
        print(var)
        yield "constant yield"


@contextlib.contextmanager
def good_cm_finally():
    contextvar = "acquired context"
    print("good cm enter")
    try:
        yield contextvar
    finally:
        print("good cm exit")


def good_cm_finally_genfunc():
    with good_cm_finally() as context:
        yield context * 2


@contextlib.contextmanager
def good_cm_no_cleanup():
    contextvar = "acquired context"
    print("cm enter")
    yield contextvar


def good_cm_no_cleanup_genfunc():
    with good_cm_no_cleanup() as context:
        yield context * 2



"""
Test for names within keyword and position only function

This test is 3.8+ as the columns are not correctly identified
by the ast parser < 3.8
"""
# pylint: disable=unused-argument,too-many-arguments


def name(
    some_thing_long_but_okay,
    not_okay_łol,  # [non-ascii-name]
    not_okay_defaułt=None,  # [non-ascii-name]
    /,
    p_or_kw_okay=None,
    p_or_kw_not_økay=None,  # [non-ascii-name]
    *,
    kw_arg_okay,
    kw_arg_not_økay,  # [non-ascii-name]
):
    """
    Do something!
    """
    return "Foobar"



"""Tests for missing-param-doc and missing-type-doc for non-specified style docstrings
with accept-no-param-doc = no and no-docstring-rgx = ^$
"""
# pylint: disable=invalid-name, unused-argument, too-few-public-methods, function-redefined
# pylint: disable=missing-class-docstring


class MyClass:
    """test_all_docstring_rgx
    Function that matches "check all functions" 'no-docstring-rgx' config option
    No error message is emitted.
    """

    def __init__(self, my_param: int) -> None:
        """
        My init docstring
        :param my_param: My first param
        """


# test_fail_empty_docstring_rgx
# Function that matches "check all functions" 'no-docstring-rgx' config option
# An error message is emitted.
class MyClass:
    def __init__(self, my_param: int) -> None:  # [missing-param-doc]
        """
        My init docstring
        """



# pylint: disable=missing-docstring,invalid-name,too-few-public-methods
class A:
    myfield: int

class B(A):
    pass

class C:
    pass

class D(C, B):
    pass


a = A()
print(a.myfield)

b = B()
print(b.myfield)

d = D()
print(d.myfield)

c = C()
print(c.myfield)  # [no-member]



"""Tests for logging-not-lazy and the logging-modules option."""
# pylint: disable=import-error, consider-using-f-string

from my import logging as blogging

blogging.warn("%s" % "%s")  # [logging-not-lazy]



"""Test case for typevar-name-incorrect-variance with default settings"""
# pylint: disable=too-few-public-methods,line-too-long
from typing import TypeVar
import typing_extensions as te

# PascalCase names with prefix
GoodNameT = TypeVar("GoodNameT")
_T = TypeVar("_T")
_GoodNameT = TypeVar("_GoodNameT")
__GoodNameT = TypeVar("__GoodNameT")
GoodNameWithoutContra = TypeVar(  # [typevar-name-incorrect-variance]
    "GoodNameWithoutContra", contravariant=True
)
GoodNameT_co = TypeVar("GoodNameT_co", covariant=True)
GoodNameT_contra = TypeVar("GoodNameT_contra", contravariant=True)
GoodBoundNameT = TypeVar("GoodBoundNameT", bound=int)

# Some of these will create a RunTime error but serve as a regression test
T = TypeVar(  # [typevar-double-variance,typevar-name-incorrect-variance]
    "T", covariant=True, contravariant=True
)
T = TypeVar("T", covariant=False, contravariant=False)
T_co = TypeVar("T_co", covariant=True, contravariant=True) # [typevar-double-variance,typevar-name-incorrect-variance]
T_contra = TypeVar(  # [typevar-double-variance,typevar-name-incorrect-variance]
    "T_contra", covariant=True, contravariant=True
)
T_co = TypeVar("T_co", covariant=True, contravariant=False)
T_contra = TypeVar("T_contra", covariant=False, contravariant=True)

# PascalCase names without prefix
AnyStr = TypeVar("AnyStr")
DeviceTypeT = TypeVar("DeviceTypeT")
HVACModeT = TypeVar("HVACModeT")
TodoT = TypeVar("TodoT")
TypeT = TypeVar("TypeT")
_IPAddress = TypeVar("_IPAddress")
CALLABLE_T = TypeVar("CALLABLE_T")  # [invalid-name]
DeviceType = TypeVar("DeviceType")  # [invalid-name]
IPAddressU = TypeVar("IPAddressU")  # [invalid-name]

# Wrong prefix
TAnyStr = TypeVar("TAnyStr")  # [invalid-name]

# camelCase names with prefix
badName = TypeVar("badName")  # [invalid-name]
badName_co = TypeVar("badName_co", covariant=True)  # [invalid-name]
badName_contra = TypeVar("badName_contra", contravariant=True)  # [invalid-name]

# PascalCase names with lower letter prefix in tuple assignment
(
    a_BadName,  # [invalid-name]
    a_BadNameWithoutContra,  # [invalid-name, typevar-name-incorrect-variance]
) = TypeVar("a_BadName"), TypeVar("a_BadNameWithoutContra", contravariant=True)
GoodName_co, a_BadName_contra = TypeVar(  # [invalid-name]
    "GoodName_co", covariant=True
), TypeVar("a_BadName_contra", contravariant=True)
GoodName_co, VAR = TypeVar("GoodName_co", covariant=True), "a string"


# -- typing_extensions.TypeVar --
GoodNameT = te.TypeVar("GoodNameT")
GoodNameT_co = te.TypeVar("GoodNameT_co", covariant=True)
badName = te.TypeVar("badName")  # [invalid-name]
T_co = te.TypeVar("T_co", covariant=True, contravariant=True)  # [typevar-double-variance,typevar-name-incorrect-variance]



# pylint: disable-msg=missing-docstring,too-few-public-methods,using-constant-test  # [deprecated-pragma]

class Class:
    @property
    @classmethod
    def func(cls):
        pass


if Class.func:
    pass



# pylint: disable=missing-docstring, comparison-with-itself, invalid-name

while 2 == 2:  # [comparison-of-constants]
    pass

while 2 > 2:  # [comparison-of-constants]
    pass

n = 2
if 2 != n:
    pass

if n != 1 + 1:
    pass

if True == True:  # [comparison-of-constants, singleton-comparison]
    pass

CONST = 24


if  0 < CONST < 42:
    pass

if  0 < n < 42:
    pass

if  True == n != 42:
    pass

if  0 == n != 42:
    pass


print(0 < n < 42)
print(0 <= n < 42 )
print(n < 1 < n*42 < 42)
print(42> n <= 0)
print(0 == n > 42)



"""Checks that a subclass does not redefine a slot which has been defined in a parent class."""

# pylint: disable=too-few-public-methods, invalid-slots-object

from collections import deque


class Base:
    """Class defining the `a`, `b` & `deque.__name__` slots"""
    __slots__ = ("a", "b", deque.__name__)


class Subclass1(Base):
    """Redefining the `a` & `deque.__name__` slots & adding the `d` & `e` slots"""
    __slots__ = ("a", deque.__name__, "d", "e")  # [redefined-slots-in-subclass]


class Subclass2(Base):
    """Adding the `f`, `g` & `h` slots"""
    __slots__ = ("f", "g", "h")


class Base2:
    """Class defining the `i`, `j` & `k` slots"""
    __slots__ = ("i", "j", "k")


class Subclass3(Base, Base2):
    """Adding the `l`, `m`, `n` slots
       Redefining the `a`, `b`, & `c` slot already defined in `Base`
       Redefining the `i`, `j`, `k` slot already defined in `Base2`
    """
    __slots__ = ("a", "b", "c", "i", "j", "k", "l", "m", "n")  # [redefined-slots-in-subclass]


# https://github.com/pylint-dev/pylint/issues/6100
class MyClass:
    """No crash when the type of the slot is not a Const or a str"""
    __slots__ = [str]



"""Tests for missing-raises-doc and missing-raises-type-doc for Google style docstrings"""
# pylint: disable=function-redefined, invalid-name, undefined-variable, missing-function-docstring
# pylint: disable=unused-argument, import-outside-toplevel, import-error, try-except-raise, too-few-public-methods


def test_find_missing_google_raises(self):  # [missing-raises-doc]
    """This is a Google docstring.

    Raises:
        NameError: Never
    """
    raise RuntimeError("hi")
    raise NameError("hi")  # [unreachable]


def test_find_google_attr_raises_exact_exc(self):
    """This is a google docstring.

    Raises:
        re.error: Sometimes
    """
    import re

    raise re.error("hi")


def test_find_google_attr_raises_substr_exc(self):
    """This is a google docstring.

    Raises:
        re.error: Sometimes
    """
    from re import error

    raise error("hi")


def test_find_valid_missing_google_attr_raises(self):  # [missing-raises-doc]
    """This is a google docstring.

    Raises:
        re.anothererror: Sometimes
    """
    from re import error

    raise error("hi")


def test_find_invalid_missing_google_attr_raises(self):
    """This is a google docstring.
    pylint allows this to pass since the comparison between Raises and
    raise are based on the class name, not the qualified name.

    Raises:
        bogusmodule.error: Sometimes
    """
    from re import error

    raise error("hi")


def test_google_raises_local_reference(self):
    """This is a google docstring.
    pylint allows this to pass since the comparison between Raises and
    raise are based on the class name, not the qualified name.

    Raises:
        .LocalException: Always
    """
    from neighbor_module import LocalException

    raise LocalException("hi")


def test_find_all_google_raises(self):
    """This is a Google docstring.

    Raises:
        RuntimeError: Always
        NameError: Never
    """
    raise RuntimeError("hi")
    raise NameError("hi")  # [unreachable]


def test_find_multiple_google_raises(self):
    """This is a Google docstring.

    Raises:
        RuntimeError: Always
        NameError, OSError, ValueError: Never
    """
    raise RuntimeError("hi")
    raise NameError("hi")  # [unreachable]
    raise OSError(2, "abort!")  # [unreachable]
    raise ValueError("foo")  # [unreachable]


def test_find_rethrown_google_raises(self):  # [missing-raises-doc]
    """This is a Google docstring.

    Raises:
        NameError: Sometimes
    """
    try:
        fake_func()
    except RuntimeError:
        raise

    raise NameError("hi")


def test_find_rethrown_google_multiple_raises(self):  # [missing-raises-doc]
    """This is a Google docstring.

    Raises:
        NameError: Sometimes
    """
    try:
        fake_func()
    except (RuntimeError, ValueError):
        raise

    raise NameError("hi")


def test_ignores_caught_google_raises(self):
    """This is a Google docstring.

    Raises:
        NameError: Sometimes
    """
    try:
        raise RuntimeError("hi")
    except RuntimeError:
        pass

    raise NameError("hi")


class Foo:
    """test_finds_missing_raises_from_setter_google
    Example of a setter having missing raises documentation in
    the Google style docstring of the property
    """

    @property
    def foo_method(self):  # [missing-raises-doc]
        """int: docstring

        Include a "Raises" section so that this is identified
        as a Google docstring and not a Numpy docstring.

        Raises:
            RuntimeError: Always
        """
        raise RuntimeError()
        return 10  # [unreachable]

    @foo_method.setter
    def foo_method(self, value):
        print(self)
        raise AttributeError()


class Foo:
    """test_finds_missing_raises_from_setter_google_2
    Example of a setter having missing raises documentation in
    its own Google style docstring of the property.
    """

    @property
    def foo_method(self):
        """int: docstring ...

        Raises:
            RuntimeError: Always
        """
        raise RuntimeError()
        return 10  # [unreachable]

    @foo_method.setter
    def foo_method(self, value):  # [missing-raises-doc]
        """setter docstring ...

        Raises:
            RuntimeError: Never
        """
        print(self)
        if True:  # [using-constant-test]
            raise AttributeError()
        raise RuntimeError()



def teacher_greeting(names):
    greetings = []
    for name in names:

        def greet():
            # do something
            print(f"Hello, {name}!")  # [cell-var-from-loop]

        if name.isalpha():
            greetings.append(greet)

    for greet in greetings:
        # the "name" variable is evaluated when the function is called here,
        # which is the last value it had in the loop - "Not-A-Name"
        greet()


teacher_greeting(["Graham", "John", "Terry", "Eric", "Terry", "Michael"])
# "Hello, Michael!"
# "Hello, Michael!"
# "Hello, Michael!"
# "Hello, Michael!"
# "Hello, Michael!"



class Orange:  # [duplicate-code]
    def __init__(self):
        self.remaining_bites = 3

    def take_bite(self):
        if self.remaining_bites > 0:
            print("You take a bite of the apple.")
            self.remaining_bites -= 1
        else:
            print("The orange is already eaten up!")

    def eaten_by_animal(self, animal):
        if animal == "cat":
            raise ValueError("A cat would never do that !")
        self.remaining_bites = 0
        print("The orange has been eaten by an animal.")



# pylint: disable=fixme,logging-too-many-args,logging-fstring-interpolation,missing-docstring,no-else-return
# pylint: disable=too-few-public-methods

# Disabled because of a bug with pypy 3.8 see
# https://github.com/pylint-dev/pylint/pull/7918#issuecomment-1352737369
# pylint: disable=multiple-statements

from abc import ABCMeta, abstractmethod
import dataclasses as dc
from typing import Any, Dict

@dc.dataclass(frozen=True)
class DeploymentState(metaclass=ABCMeta):
    type: str

    @abstractmethod
    def to_dict(self) -> Dict:
        """
        Serializes given DeploymentState instance to Dict.
        :return:
        """

@dc.dataclass(frozen=True)
class DeploymentStateEcs(DeploymentState):
    blue: Any
    green: Any
    candidate: Any

    def to_dict(self) -> Dict:
        return {
            'type': self.type,
            'blue': dc.asdict(self.blue),
            'green': dc.asdict(self.green),
            'candidate': self.candidate.value,
        }

@dc.dataclass(frozen=True)
class DeploymentStateLambda(DeploymentState):
    current: Any
    candidate: Any

    def to_dict(self) -> Dict:
        return {
            'type': self.type,
            'current': dc.asdict(self.current),
            'candidate': dc.asdict(self.candidate) if self.candidate else None,
        }



"""Tests for missing-param-doc and missing-type-doc for non-specified style docstrings
with accept-no-param-doc = yes
"""
# pylint: disable=invalid-name, unused-argument


def test_tolerate_no_param_documentation_at_all(x, y):
    """Example of a function with no parameter documentation at all

    No error message is emitted.

    missing parameter documentation"""



""" Test that superfluous else return are detected. """


# pylint:disable=invalid-name,missing-docstring,unused-variable
def foo1(x, y, z):
    if x:  # [no-else-return]
        a = 1
        return y
    else:
        b = 2
        return z


def foo2(x, y, w, z):
    if x:  # [no-else-return]
        a = 1
        return y
    elif z:
        b = 2
        return w
    else:
        c = 3
        return z


def foo3(x, y, z):
    if x:
        a = 1
        if y:  # [no-else-return]
            b = 2
            return y
        else:
            c = 3
            return x
    else:
        d = 4
        return z


def foo4(x, y):
    if x:   # [no-else-return]
        if y:
            a = 4
        else:
            b = 2
        return
    else:
        c = 3
    return


def foo5(x, y, z):
    if x:   # [no-else-return]
        if y:
            a = 4
        else:
            b = 2
        return
    elif z:
        c = 2
    else:
        c = 3
    return


def foo6(x, y):
    if x:
        if y:   # [no-else-return]
            a = 4
            return
        else:
            b = 2
    else:
        c = 3
    return


def bar1(x, y, z):
    if x:
        return y
    return z


def bar2(w, x, y, z):
    if x:
        return y
    if z:
        a = 1
    else:
        return w
    return None


def bar3(x, y, z):
    if x:
        if z:
            return y
    else:
        return z
    return None


def bar4(x):
    if x:  # [no-else-return]
        return True
    else:
        try:
            return False
        except ValueError:
            return None

# pylint: disable = bare-except
def try_one_except() -> bool:
    try:  # [no-else-return]
        print('asdf')
    except:
        print("bad")
        return False
    else:
        return True


def try_multiple_except() -> bool:
    try:  # [no-else-return]
        print('asdf')
    except TypeError:
        print("bad")
        return False
    except ValueError:
        print("bad second")
        return False
    else:
        return True

def try_not_all_except_return() -> bool:  # [inconsistent-return-statements]
    try:
        print('asdf')
    except TypeError:
        print("bad")
        return False
    except ValueError:
        val = "something"
    else:
        return True

# pylint: disable = raise-missing-from
def try_except_raises() -> bool:
    try:  # [no-else-raise]
        print('asdf')
    except:
        raise ValueError
    else:
        return True

def try_except_raises2() -> bool:
    try:  # [no-else-raise]
        print('asdf')
    except TypeError:
        raise ValueError
    except ValueError:
        raise TypeError
    else:
        return True

def test() -> bool:  # [inconsistent-return-statements]
    try:
        print('asdf')
    except RuntimeError:
        return False
    finally:
        print("in finally")


def try_finally_return() -> bool:  # [inconsistent-return-statements]
    try:
        print('asdf')
    except RuntimeError:
        return False
    else:
        print("inside else")
    finally:
        print("in finally")

def try_finally_raise():
    current_tags = {}
    try:
        yield current_tags
    except Exception:
        current_tags["result"] = "failure"
        raise
    else:
        current_tags["result"] = "success"
    finally:
        print("inside finally")



"""Example checker detecting deprecated functions/methods. Following example searches for usages of
deprecated function `deprecated_function` and deprecated method `MyClass.deprecated_method`
from module mymodule:

.. code-block:: console
    $ cat mymodule.py
    def deprecated_function():
        pass

    def myfunction(arg0, arg1, deprecated_arg1=None, arg2='foo', arg3='bar', deprecated_arg2='spam'):
        pass

    class MyClass:
        def deprecated_method(self):
            pass

        def mymethod(self, arg0, arg1, deprecated1=None, arg2='foo', deprecated2='bar', arg3='spam'):
            pass

    $ cat mymain.py
    from mymodule import deprecated_function, myfunction, MyClass

    deprecated_function()
    myfunction(0, 1, 'deprecated_arg1', deprecated_arg2=None)
    MyClass().deprecated_method()
    MyClass().mymethod(0, 1, deprecated1=None, deprecated2=None)

    $ pylint --load-plugins=deprecation_checker mymain.py
    ************* Module mymain
    mymain.py:3:0: W1505: Using deprecated method deprecated_function() (deprecated-method)
    mymain.py:4:0: W1511: Using deprecated argument deprecated_arg1 of method myfunction() (deprecated-argument)
    mymain.py:4:0: W1511: Using deprecated argument deprecated_arg2 of method myfunction() (deprecated-argument)
    mymain.py:5:0: W1505: Using deprecated method deprecated_method() (deprecated-method)
    mymain.py:6:0: W1511: Using deprecated argument deprecated1 of method mymethod() (deprecated-argument)
    mymain.py:6:0: W1511: Using deprecated argument deprecated2 of method mymethod() (deprecated-argument)

    ------------------------------------------------------------------
    Your code has been rated at 2.00/10 (previous run: 2.00/10, +0.00)
"""

from __future__ import annotations

from typing import TYPE_CHECKING

from pylint.checkers import BaseChecker, DeprecatedMixin

if TYPE_CHECKING:
    from pylint.lint import PyLinter


class DeprecationChecker(DeprecatedMixin, BaseChecker):
    """Class implementing deprecation checker."""

    # DeprecatedMixin class is Mixin class implementing logic for searching deprecated methods and functions.
    # The list of deprecated methods/functions is defined by the implementing class via
    # deprecated_methods callback. DeprecatedMixin class is overriding attributes of BaseChecker hence must
    # be specified *before* BaseChecker in list of base classes.

    # The name defines a custom section of the config for this checker.
    name = "deprecated"

    # Register messages emitted by the checker.
    msgs = {
        **DeprecatedMixin.DEPRECATED_METHOD_MESSAGE,
        **DeprecatedMixin.DEPRECATED_ARGUMENT_MESSAGE,
    }

    def deprecated_methods(self) -> set[str]:
        """Callback method called by DeprecatedMixin for every method/function found in the code.

        Returns:
            collections.abc.Container of deprecated function/method names.
        """
        return {"mymodule.deprecated_function", "mymodule.MyClass.deprecated_method"}

    def deprecated_arguments(self, method: str) -> tuple[tuple[int | None, str], ...]:
        """Callback returning the deprecated arguments of method/function.

        Returns:
            collections.abc.Iterable in form:
                ((POSITION1, PARAM1), (POSITION2: PARAM2) ...)
            where
                * POSITIONX - position of deprecated argument PARAMX in function definition.
                  If argument is keyword-only, POSITIONX should be None.
                * PARAMX - name of the deprecated argument.
        """
        if method == "mymodule.myfunction":
            # myfunction() has two deprecated arguments:
            # * deprecated_arg1 defined at 2nd position and
            # * deprecated_arg2 defined at 5th position.
            return ((2, "deprecated_arg1"), (5, "deprecated_arg2"))
        if method == "mymodule.MyClass.mymethod":
            # mymethod() has two deprecated arguments:
            # * deprecated1 defined at 2nd position and
            # * deprecated2 defined at 4th position.
            return ((2, "deprecated1"), (4, "deprecated2"))
        return ()


def register(linter: PyLinter) -> None:
    linter.register_checker(DeprecationChecker(linter))



r"""A raw docstring.
"""


def look_busy():
    xxxx = 1  # pylint: disable=duplicate-code
    yyyy = 2  # pylint: disable=duplicate-code
    zzzz = 3  # pylint: disable=duplicate-code
    wwww = 4  # pylint: disable=duplicate-code
    vvvv = xxxx + yyyy + zzzz + wwww  # pylint: disable=duplicate-code
    return vvvv  # pylint: disable=duplicate-code



# pylint: disable=missing-docstring,expression-not-assigned,too-few-public-methods,pointless-statement


class Unhashable:
    __hash__ = list.__hash__

# Subscripts
{}[[1, 2, 3]] # [unhashable-member]
{}[{}] # [unhashable-member]
{}[Unhashable()] # [unhashable-member]
{'foo': 'bar'}['foo']
{'foo': 'bar'}[42]

# Keys
{[1, 2, 3]: "tomato"}  # [unhashable-member]
{
    [1, 2, 3]: "tomato",  # [unhashable-member]
    [4, 5, 6]: "celeriac",  # [unhashable-member]
}
{[1, 2, 3]}  # [unhashable-member]
{"tomato": "tomahto"}
{dict: {}}
{lambda x: x: "tomato"}  # pylint: disable=unnecessary-lambda


class FromDict(dict):
    ...

{FromDict: 1}



def myfunc(shall_continue: bool, shall_exit: bool):
    if shall_continue:
        if input("Are you sure?") == "y":
            print("Moving on.")
    elif shall_exit:  # [confusing-consecutive-elif]
        print("Exiting.")



def divide_x_by_y(x: float, y: float):
    try:
        print(x / y)
    except ArithmeticError as e:
        print(
            f"There was an OverflowError, a ZeroDivisionError or a FloatingPointError: {e}"
        )



"""Homonym between filtered comprehension and assignment in except block."""
# pylint: disable=broad-exception-raised

def func():
    """https://github.com/pylint-dev/pylint/issues/5586"""
    try:
        print(value for value in range(1 / 0) if isinstance(value, int))
    except ZeroDivisionError:
        value = 1
        print(value)


def func2():
    """Same, but with attribute access."""
    try:
        print(value for value in range(1 / 0) if isinstance(value.num, int))
    except ZeroDivisionError:
        value = 1
        print(value)


def func3():
    """Same, but with no call."""
    try:
        print(value for value in range(1 / 0) if value)
    except ZeroDivisionError:
        value = 1
        print(value)


def func4():
    """https://github.com/pylint-dev/pylint/issues/6035"""
    assets = [asset for asset in range(3) if asset.name == "filename"]

    try:
        raise ValueError
    except ValueError:
        asset = assets[0]
        print(asset)


def func5():
    """Similar, but with subscript notation"""
    results = {}
    # pylint: disable-next=consider-using-dict-items
    filtered =  [k for k in results if isinstance(results[k], dict)]

    try:
        1 / 0
    except ZeroDivisionError:
        k = None
        print(k, filtered)


def func6(data, keys):
    """Similar, but with a subscript in a key-value pair rather than the test
    See https://github.com/pylint-dev/pylint/issues/6069"""
    try:
        results = {key: data[key] for key in keys}
    except KeyError as exc:
        key, *_ = exc.args
        raise Exception(f"{key} not found") from exc

    return results


def func7():
    """Similar, but with a comparison"""
    bools = [str(i) == i for i in range(3)]

    try:
        1 / 0
    except ZeroDivisionError:
        i = None
        print(i, bools)


def func8():
    """Similar, but with a container"""
    pairs = [(i, i) for i in range(3)]

    try:
        1 / 0
    except ZeroDivisionError:
        i = None
        print(i, pairs)


# Module level cases

module_ints = [j | j for j in range(3)]
try:
    1 / 0
except ZeroDivisionError:
    j = None
    print(j, module_ints)



"""
Regression test for `no-member`.
See: https://github.com/pylint-dev/pylint/issues/2964
"""

# pylint: disable=missing-class-docstring,too-few-public-methods
# pylint: disable=unused-private-member,protected-access


class Node:
    def __init__(self, name, path=()):
        """
        Initialize self with "name" string and the tuple "path" of its parents.
        "self" is added to the tuple as its last item.
        """
        self.__name = name
        self.__path = path + (self,)

    def get_full_name(self):
        """
        A `no-member` message was emitted:
        nodes.py:17:24: E1101: Instance of 'tuple' has no '__name' member (no-member)
        """
        return ".".join(node.__name for node in self.__path)



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

"""Test for the 'pylint-config generate' command."""


import warnings

import pytest
from pytest import CaptureFixture

from pylint.lint.run import _PylintConfigRun as Run


def test_pylint_config_main_messages(capsys: CaptureFixture[str]) -> None:
    """Check that the help messages are displayed correctly."""
    with warnings.catch_warnings():
        warnings.filterwarnings("ignore", message="NOTE:.*", category=UserWarning)
        Run([], exit=False)
        captured = capsys.readouterr()
        assert captured.out.startswith("usage: pylint-config [options]")
        assert "--interactive" not in captured.out

        Run(["-h"], exit=False)
        captured_two = capsys.readouterr()
        assert captured_two.out == captured.out

        with pytest.raises(SystemExit):
            Run(["generate", "-h"])
        captured = capsys.readouterr()
        assert captured.out.startswith("usage: pylint-config [options] generate")
        assert "--interactive" in captured.out

        with pytest.raises(SystemExit) as ex:
            Run(["generate"])
        captured_two = capsys.readouterr()
        assert captured_two.out == captured.out
        # This gets auto-raised by argparse to be 0.
        assert ex.value.code == 32

        with pytest.raises(SystemExit) as ex:
            Run(["-h"])
        assert ex.value.code == 32



"""Test for a regression with Enums not being recognized when imported with an alias.

Reported in https://github.com/pylint-dev/pylint/issues/5776
"""

from enum import Enum as PyEnum


class MyEnum(PyEnum):
    """My enum"""

    ENUM_KEY = "enum_value"


print(MyEnum.ENUM_KEY.value)



"""Tests for undefined-loop-variable with assignment expressions"""


def walrus_in_comprehension_test(container):
    """https://github.com/pylint-dev/pylint/issues/7222"""
    for something in container:
        print(something)
    print([my_test for something in container if (my_test := something)])



"""Fixture for testing missing documentation in docparams (Python >=3.8 only)."""


def differing_param_doc_pos_only(par1: int, /) -> int:  # [differing-param-doc]
    """This is a test docstring documenting one non-existing param

    :param par1: some param
    :param param: some param
    :return: the sum of the params
    """

    return par1


def missing_type_doc_pos_only(par1, /) -> int:  # [missing-type-doc]
    """This is a test docstring params where the type is not specified

    :param par1: some param
    :return: the param
    """

    return par1


def params_are_documented(par1: int, /, par2: int, *, par3: int) -> int:
    """This is a test docstring params where nothing is raised as it is all documented

    :param par1: some param
    :param par2: some other param
    :param par3: some other param
    :return: the sum of params
    """

    return par1 + par2 + par3



#pylint: disable=missing-module-docstring

def foobar1(arg1, arg2): #[missing-any-param-doc]
    """function foobar ...
    """
    print(arg1, arg2)

def foobar2(arg1, arg2): #[missing-any-param-doc]
    """function foobar ...
    Parameters
    ----------
    """
    print(arg1, arg2)

def foobar3(arg1, arg2, arg3): #[missing-param-doc, missing-type-doc]
    """function foobar ...
    Parameters
    ----------
    arg1: int
    arg3: float
    """
    print(arg1, arg2, arg3)

def foobar4(arg1, arg2): #[missing-param-doc, missing-type-doc]
    """function foobar ...
    Parameters
    ----------
    arg1: int
        description
    """
    print(arg1, arg2)

def foobar5(arg1, arg2): #[missing-type-doc]
    """function foobar ...
    Parameters
    ----------
    arg1:
        description
    arg2: str
    """
    print(arg1, arg2)

def foobar6(arg1, arg2, arg3): #[missing-param-doc, missing-type-doc]
    """function foobar ...
    Parameters
    ----------
    arg1: int
        description
    arg2: int
    """
    print(arg1, arg2, arg3)

def foobar7(arg1, arg2): #[missing-any-param-doc]
    """function foobar ...
    Parameters
    ----------
    arg1
    """
    print(arg1, arg2)

def foobar8(arg1): #[missing-any-param-doc]
    """function foobar"""

    print(arg1)

def foobar9(arg1, arg2, arg3):
    """function foobar ...
    Parameters
    ----------
    arg1: int
    arg2: int
    arg3: str
    """
    print(arg1, arg2, arg3)

def foobar10(arg1, arg2, arg3): #[missing-type-doc]
    """function foobar ...
    Parameters
    ----------
    arg1:
        desc1
    arg2: int
    arg3:
        desc3
    """
    print(arg1, arg2, arg3)

def foobar11(arg1, arg2): #[missing-any-param-doc]
    """function foobar ...
    Args
    ----------
    arg1
    arg2
    """
    print(arg1, arg2)

def foobar12(arg1, arg2, arg3): #[missing-param-doc, missing-type-doc]
    """function foobar ...
    Args
    ----------
    arg1: int
    arg2:
        does something
    arg3
    """
    print(arg1, arg2, arg3)

def foobar13(arg1, *args, arg3=";"):
    """Description of the function

    Parameters
    ----------
    arg1 : str
        Path to the input.
    *args :
        Relevant parameters.
    arg3 : str, optional
        File separator.
    """
    print(arg1, args, arg3)

def foobar14(arg1, *args):
    """Description of the function

    Parameters
    ----------
    arg1 : str
        Path to the input.
    *args :
        Relevant parameters.
    """
    print(arg1, args)

def foobar15(*args):
    """Description of the function

    Parameters
    ----------
    *args :
        Relevant parameters.
    """
    print(args)


def foobar16(one: int, two: str, three: float) -> int:
    """Description of the function

    Args:
        one: A number.
        two: Another number.
        three: Yes another number.

    Returns:
        The number one.
    """
    print(one, two, three)
    return 1


def foobar17(one, two, three):
    # type: (int, str, float) -> int
    """Description of the function

    Args:
        one: A number.
        two: Another number.
        three: Yes another number.

    Returns:
        The number one.
    """
    print(one, two, three)
    return 1


def foobar18(
    one,  # type: int
    two,  # type: str
    three,  # type: float
):
    # type: (...) -> int
    """Description of the function

    Args:
        one: A number.
        two: Another number.
        three: Yes another number.

    Returns:
        The number one.
    """
    print(one, two, three)
    return 1


def foobar19(one, two, **kwargs):
    # type: (int, str, float) -> int
    """Description of the function

    Args:
        one: A number.
        two: Another number.
        kwargs: More numbers.

    Returns:
        The number one.
    """
    print(one, two, kwargs)
    return 1



import dataclasses


@dataclasses.dataclass
class Worm:
    name: str
    type: str
    color: str


class Fruit:
    def __init__(self):
        self.name = "Little Apple"
        self.color = "Bright red"
        self.vitamins = ["A", "B1"]
        self.antioxidants = None
        self.worms = [
            Worm(name="Jimmy", type="Codling Moths", color="light brown"),
            Worm(name="Kim", type="Apple maggot", color="Whitish"),
        ]



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

"""Test for the 'pylint-config generate' command."""


import warnings

import pytest
from pytest import CaptureFixture

from pylint import _run_pylint_config


def test_invocation_of_pylint_config(capsys: CaptureFixture[str]) -> None:
    """Check that the help messages are displayed correctly."""
    with warnings.catch_warnings():
        warnings.filterwarnings("ignore", message="NOTE:.*", category=UserWarning)
        with pytest.raises(SystemExit) as ex:
            _run_pylint_config([""])
        captured = capsys.readouterr()
        assert captured.err.startswith("usage: pylint-config [options]")
        assert ex.value.code == 2



# pylint: disable=too-few-public-methods, super-init-not-called
# pylint: disable=no-classmethod-decorator
"""Test external access to protected class members."""

class MyClass:
    """Class with protected members."""
    _cls_protected = 5

    def __init__(self, other):
        MyClass._cls_protected = 6
        self._protected = 1
        self.public = other
        self.attr = 0

    def test(self):
        """Docstring."""
        self._protected += self._cls_protected
        print(self.public._haha)  # [protected-access]

    def clsmeth(cls):
        """Docstring."""
        cls._cls_protected += 1
        print(cls._cls_protected)
    clsmeth = classmethod(clsmeth)

    def _private_method(self):
        """Doing nothing."""


class Subclass(MyClass):
    """Subclass with protected members."""

    def __init__(self):
        MyClass._protected = 5
        super()._private_method()

INST = Subclass()
INST.attr = 1
print(INST.attr)
INST._protected = 2  # [protected-access]
print(INST._protected)  # [protected-access]
INST._cls_protected = 3  # [protected-access]
print(INST._cls_protected)  # [protected-access]


class Issue1031:
    """Test for GitHub issue 1031"""
    _attr = 1

    def correct_access(self):
        """Demonstrates correct access"""
        return type(self)._attr

    def incorrect_access(self):
        """Demonstrates incorrect access"""
        if self._attr == 1:
            return type(INST)._protected  # [protected-access]
        return None


class Issue1802:
    """Test for GitHub issue 1802"""
    def __init__(self, value):
        self._foo = value
        self.__private = 2 * value

    def __eq__(self, other):
        """Test a correct access as the access to protected member is in a special method"""
        if isinstance(other, self.__class__):
            answer = self._foo == other._foo
            return answer and self.__private == other.__private  # [protected-access]
        return False

    def not_in_special(self, other):
        """
        Test an incorrect access as the access to protected member is not inside a special method
        """
        if isinstance(other, self.__class__):
            return self._foo == other._foo  # [protected-access]
        return False

    def __le__(self, other):
        """
        Test a correct access as the access to protected member
        is inside a special method even if it is deeply nested
        """
        if 2 > 1:  # pylint: disable=comparison-of-constants
            if isinstance(other, self.__class__):
                if "answer" == "42":  # pylint: disable=comparison-of-constants
                    return self._foo == other._foo
        return False

    def __fake_special__(self, other):
        """
        Test an incorrect access as the access
        to protected member is not inside a licit special method
        """
        if isinstance(other, self.__class__):
            return self._foo == other._foo  # [protected-access]
        return False


class Issue1159OtherClass:
    """Test for GitHub issue 1159"""

    _foo = 0

    def __init__(self):
        self._bar = 0


class Issue1159:
    """Test for GitHub issue 1159"""

    _foo = 0

    def __init__(self):
        self._bar = 0

    @classmethod
    def access_cls_attr(cls):
        """
        Access to protected class members inside class methods is OK.
        """

        _ = cls._foo

    @classmethod
    def assign_cls_attr(cls):
        """
        Assignment to protected class members inside class methods is OK.
        """

        cls._foo = 1

    @classmethod
    def access_inst_attr(cls):
        """
        Access to protected instance members inside class methods is OK.
        """

        instance = cls()
        _ = instance._bar

    @classmethod
    def assign_inst_attr(cls):
        """
        Assignment to protected members inside class methods is OK.
        """

        instance = cls()
        instance._bar = 1

    @classmethod
    def access_other_attr(cls):
        """
        Access to protected instance members of other classes is not OK.
        """

        instance = Issue1159OtherClass()
        instance._bar = 3  # [protected-access]
        _ = instance._foo  # [protected-access]


class Issue1159Subclass(Issue1159):
    """Test for GitHub issue 1159"""

    @classmethod
    def access_inst_attr(cls):
        """
        Access to protected instance members inside class methods is OK.
        """

        instance = cls()
        _ = instance._bar

    @classmethod
    def assign_inst_attr(cls):
        """
        Assignment to protected instance members inside class methods is OK.
        """

        instance = cls()
        instance._bar = 1

    @classmethod
    def access_missing_member(cls):
        """
        Access to unassigned members inside class methods is not OK.
        """

        instance = cls()
        _ = instance._baz  # [no-member,protected-access]

    @classmethod
    def assign_missing_member(cls):
        """
        Defining attributes outside init is still not OK.
        """

        instance = cls()
        instance._qux = 1  # [attribute-defined-outside-init]

    @classmethod
    def access_other_attr(cls):
        """
        Access to protected instance members of other classes is not OK.
        """

        instance = Issue1159OtherClass()
        instance._bar = 3  # [protected-access]
        _ = instance._foo  # [protected-access]


class Issue3066:
    """Test for GitHub issue 3066
    Accessing of attributes/methods of inner and outer classes
    https://github.com/pylint-dev/pylint/issues/3066"""

    attr = 0
    _attr = 1

    @staticmethod
    def _bar(i):
        """Docstring."""

    @staticmethod
    def foobar(i):
        """Test access from outer class"""
        Issue3066._attr = 2
        Issue3066.Aclass._attr = "y"  # [protected-access]
        Issue3066.Aclass.Bclass._attr = "b"  # [protected-access]

        Issue3066._bar(i)
        Issue3066.Aclass._bar(i)  # [protected-access]
        Issue3066.Aclass.Bclass._bar(i)  # [protected-access]

    class Aclass:
        """Inner class for GitHub issue 3066"""

        _attr = "x"

        @staticmethod
        def foobar(i):
            """Test access from inner class"""
            Issue3066._attr = 2  # [protected-access]
            Issue3066.Aclass._attr = "y"
            Issue3066.Aclass.Bclass._attr = "b"  # [protected-access]

            Issue3066._bar(i)  # [protected-access]
            Issue3066.Aclass._bar(i)
            Issue3066.Aclass.Bclass._bar(i)  # [protected-access]

        @staticmethod
        def _bar(i):
            """Docstring."""

        class Bclass:
            """Inner inner class for GitHub issue 3066"""

            _attr = "a"

            @staticmethod
            def foobar(i):
                """Test access from inner inner class"""
                Issue3066._attr = 2  # [protected-access]
                Issue3066.Aclass._attr = "y"  # [protected-access]
                Issue3066.Aclass.Bclass._attr = "b"

                Issue3066._bar(i)  # [protected-access]
                Issue3066.Aclass._bar(i)  # [protected-access]
                Issue3066.Aclass.Bclass._bar(i)

            @staticmethod
            def _bar(i):
                """Docstring."""



def fruit_is_dangerous_for_cat(fruit: str) -> bool:
    """This list is only a silly example, don't make decision regarding your cat diet based on it."""
    return fruit in ["cherry", "grapes"]  # [use-set-for-membership]



"""Tests for missing-param-doc and missing-type-doc for Sphinx style docstrings
with accept-no-param-doc = no"""

# pylint: disable=function-redefined, invalid-name, undefined-variable, missing-class-docstring
# pylint: disable=unused-argument, too-few-public-methods, unnecessary-pass, line-too-long
# pylint: disable=missing-function-docstring, disallowed-name

from __future__ import annotations


def test_missing_func_params_in_sphinx_docstring(  # [missing-param-doc, missing-type-doc]
    x, y, z
):
    """Example of a function with missing Sphinx parameter documentation in
    the docstring

    :param x: bla

    :param int z: bar
    """
    pass


class Foo:
    def test_missing_method_params_in_sphinx_docstring(  # [missing-param-doc, missing-type-doc]
        self, x, y
    ):
        """Example of a class method with missing parameter documentation in
        the Sphinx style docstring

        missing parameter documentation

        :param x: bla
        """
        pass


def test_existing_func_params_in_sphinx_docstring(xarg, yarg, zarg, warg):
    """Example of a function with correctly documented parameters and
    return values (Sphinx style)

    :param xarg: bla xarg
    :type xarg: int

    :parameter yarg: bla yarg
    :type yarg: my.qualified.type

    :arg int zarg: bla zarg

    :keyword my.qualified.type warg: bla warg

    :return: sum
    :rtype: float
    """
    return xarg + yarg


def test_wrong_name_of_func_params_in_sphinx_docstring(  # [missing-param-doc, missing-type-doc, differing-param-doc, differing-type-doc]
    xarg, yarg, zarg
):
    """Example of functions with inconsistent parameter names in the
        signature and in the Sphinx style documentation

    :param xarg1: bla xarg
    :type xarg: int

    :param yarg: bla yarg
    :type yarg1: float

    :param str zarg1: bla zarg
    """
    return xarg + yarg


def test_wrong_name_of_func_params_in_sphinx_docstring_two(  # [differing-param-doc, differing-type-doc]
    xarg, yarg, zarg
):
    """Example of functions with inconsistent parameter names in the
        signature and in the Sphinx style documentation

    :param yarg1: bla yarg
    :type yarg1: float

    For the other parameters, see bla.
    """
    return xarg + yarg


def test_see_sentence_for_func_params_in_sphinx_docstring(xarg, yarg) -> None:
    """Example for the usage of "For the other parameters, see" to avoid
        too many repetitions, e.g. in functions or methods adhering to a
        given interface (Sphinx style)

    :param yarg: bla yarg
    :type yarg: float

    For the other parameters, see :func:`bla`
    """
    return xarg + yarg


class ClassFoo:  # [missing-param-doc, missing-type-doc]
    """test_constr_params_in_class_sphinx
    Example of a class with missing constructor parameter documentation
    (Sphinx style)

    Everything is completely analogous to functions.

    :param y: bla

    missing constructor parameter documentation
    """

    def __init__(self, x, y):
        pass


class ClassFoo:
    def __init__(self, x, y):  # [missing-param-doc, missing-type-doc]
        """test_constr_params_in_init_sphinx
        Example of a class with missing constructor parameter documentation
        (Sphinx style)

        Everything is completely analogous to functions.

        :param y: bla

        missing constructor parameter documentation
        """

        pass


class ClassFoo:  # [multiple-constructor-doc, missing-param-doc, missing-type-doc]
    """test_constr_params_in_class_and_init_sphinx
    Example of a class with missing constructor parameter documentation
    in both the init docstring and the class docstring
    (Sphinx style)

    Everything is completely analogous to functions.

    :param y: None

    missing constructor parameter documentation
    """

    def __init__(self, x, y):  # [missing-param-doc, missing-type-doc]
        """docstring foo

        :param y: bla

        missing constructor parameter documentation
        """
        pass


def test_warns_missing_args_sphinx(  # [missing-param-doc, inconsistent-return-statements]
    named_arg, *args
):
    """The docstring

    :param named_arg: Returned
    :type named_arg: object

    :returns: Maybe named_arg
    :rtype: object or None
    """
    if args:
        return named_arg


def test_warns_missing_kwargs_sphinx(  # [missing-param-doc, inconsistent-return-statements]
    named_arg, **kwargs
):
    """The docstring

    :param named_arg: Returned
    :type named_arg: object

    :returns: Maybe named_arg
    :rtype: object or None
    """
    if kwargs:
        return named_arg


def test_warns_typed_kwargs_sphinx(  # [missing-param-doc, inconsistent-return-statements]
    named_arg, **kwargs: dict[str, str]
):
    """The docstring

    :param named_arg: Returned
    :type named_arg: object

    :returns: Maybe named_arg
    :rtype: object or None
    """
    if kwargs:
        return named_arg


def test_finds_args_without_type_sphinx(  # [missing-param-doc, inconsistent-return-statements]
    named_arg, *args
):
    """The docstring

    :param named_arg: Returned
    :type named_arg: object

    :param *args: Optional arguments

    :returns: Maybe named_arg
    :rtype: object or None
    """
    if args:
        return named_arg


def test_finds_kwargs_without_type_sphinx(  # [missing-param-doc, inconsistent-return-statements]
    named_arg, **kwargs
):
    """The docstring

    :param named_arg: Returned
    :type named_arg: object

    :param **kwargs: Keyword arguments

    :returns: Maybe named_arg
    :rtype: object or None
    """
    if kwargs:
        return named_arg


def test_finds_args_without_type_sphinx(  # [inconsistent-return-statements]
    named_arg, *args
):
    r"""The Sphinx docstring
    In Sphinx docstrings asterisks should be escaped.
    See https://github.com/pylint-dev/pylint/issues/5406

    :param named_arg: Returned
    :type named_arg: object

    :param \*args: Optional arguments

    :returns: Maybe named_arg
    :rtype: object or None
    """
    if args:
        return named_arg


def test_finds_kwargs_without_type_sphinx(  # [inconsistent-return-statements]
    named_arg, **kwargs
):
    r"""The Sphinx docstring
    In Sphinx docstrings asterisks should be escaped.
    See https://github.com/pylint-dev/pylint/issues/5406

    :param named_arg: Returned
    :type named_arg: object

    :param \**kwargs: Keyword arguments

    :returns: Maybe named_arg
    :rtype: object or None
    """
    if kwargs:
        return named_arg


def test_finds_args_without_type_sphinx(  # [inconsistent-return-statements]
    named_arg, *args
):
    r"""The Sphinx docstring
    We can leave the asterisk out.

    :param named_arg: Returned
    :type named_arg: object

    :param args: Optional arguments

    :returns: Maybe named_arg
    :rtype: object or None
    """
    if args:
        return named_arg


def test_finds_kwargs_without_type_sphinx(  # [inconsistent-return-statements]
    named_arg, **kwargs
):
    r"""The Sphinx docstring
    We can leave the asterisk out.

    :param named_arg: Returned
    :type named_arg: object

    :param kwargs: Keyword arguments

    :returns: Maybe named_arg
    :rtype: object or None
    """
    if kwargs:
        return named_arg


class Foo:
    """test_finds_missing_raises_from_setter_sphinx
    Example of a setter having missing raises documentation in
    the Sphinx style docstring of the property
    """

    @property
    def foo(self):  # [missing-raises-doc]
        """docstring ...

        :type: int
        """
        return 10

    @foo.setter
    def foo(self, value):
        raise AttributeError()


class Foo:
    """test_finds_missing_raises_in_setter_sphinx
    Example of a setter having missing raises documentation in
    its own Sphinx style docstring
    """

    @property
    def foo(self):
        """docstring ...

        :type: int
        :raises RuntimeError: Always
        """
        raise RuntimeError()
        return 10  # [unreachable]

    @foo.setter
    def foo(self, value):  # [missing-raises-doc, missing-param-doc, missing-type-doc]
        """setter docstring ...

        :type: None
        """
        raise AttributeError()


class Foo:
    """test_finds_property_return_type_sphinx
    Example of a property having return documentation in
    a Sphinx style docstring
    """

    @property
    def foo(self):
        """docstring ...

        :type: int
        """
        return 10


class Foo:
    """test_finds_annotation_property_return_type_sphinx
    Example of a property having missing return documentation in
    a Sphinx style docstring
    """

    @property
    def foo(self) -> int:
        """docstring ...

        :raises RuntimeError: Always
        """
        raise RuntimeError()
        return 10  # [unreachable]


class Foo:
    def test_useless_docs_ignored_argument_names_sphinx(  # [useless-type-doc, useless-param-doc]
        self, arg, _, _ignored
    ):
        """Example of a method documenting the return type that an
        implementation should return.

        :param arg: An argument.
        :type arg: int

        :param _: Another argument.
        :type _: float

        :param _ignored: Ignored argument.
        """
        pass


def test_finds_multiple_types_sphinx_one(named_arg):
    """The Sphinx docstring

    :param named_arg: Returned
    :type named_arg: dict(str, str)

    :returns: named_arg
    :rtype: dict(str, str)
    """
    return named_arg


def test_finds_multiple_types_sphinx_two(named_arg):
    """The Sphinx docstring

    :param named_arg: Returned
    :type named_arg: dict[str, str]

    :returns: named_arg
    :rtype: dict[str, str]
    """
    return named_arg


def test_finds_multiple_types_sphinx_three(named_arg):
    """The Sphinx docstring

    :param named_arg: Returned
    :type named_arg: int or str

    :returns: named_arg
    :rtype: int or str
    """
    return named_arg


def test_finds_multiple_types_sphinx_four(named_arg):
    """The Sphinx docstring

    :param named_arg: Returned
    :type named_arg: tuple(int or str)

    :returns: named_arg
    :rtype: tuple(int or str)
    """
    return named_arg


def test_finds_multiple_types_sphinx_five(named_arg):
    """The Sphinx docstring

    :param named_arg: Returned
    :type named_arg: tuple(int) or list(int)

    :returns: named_arg
    :rtype: tuple(int) or list(int)
    """
    return named_arg


def test_finds_multiple_types_sphinx_six(named_arg):
    """The Sphinx docstring

    :param named_arg: Returned
    :type named_arg: tuple(int or str) or list(int or str)

    :returns: named_arg
    :rtype: tuple(int or str) or list(int or str)
    """
    return named_arg


def test_finds_compact_container_types_sphinx_one(named_arg):
    """The Sphinx docstring

    :param dict(str,str) named_arg: Returned

    :returns: named_arg
    :rtype: dict(str,str)
    """
    return named_arg


def test_finds_compact_container_types_sphinx_two(named_arg):
    """The Sphinx docstring

    :param dict[str,str] named_arg: Returned

    :returns: named_arg
    :rtype: dict[str,str]
    """
    return named_arg


def test_finds_compact_container_types_sphinx_three(named_arg):
    """The Sphinx docstring

    :param tuple(int) named_arg: Returned

    :returns: named_arg
    :rtype: tuple(int)
    """
    return named_arg


def test_finds_compact_container_types_sphinx_four(named_arg):
    """The Sphinx docstring

    :param list[tokenize.TokenInfo] named_arg: Returned

    :returns: named_arg
    :rtype: list[tokenize.TokenInfo]
    """
    return named_arg



"""Tests for bad string format type"""
# pylint: disable=consider-using-f-string, pointless-statement

# Test formatting of bytes
b"test".format(1, 2) # [no-member]

# Test format types
"%s" % 1
"%d" % 1
"%f" % 1
"%s" % 1
"%(key)s" % {"key": 1}
"%d" % 1
"%(key)d" % {"key": 1}
"%f" % 1
"%(key)f" % {"key": 1}
"%d" % 1.1
"%(key)d" % {"key": 1.1}
"%s" % []
"%(key)s" % {"key": []}
"%s" % None
"%(key)s" % {"key": None}

# Test incorrect format types
"%d" % "1"  # [bad-string-format-type]
"%(key)d" % {"key": "1"}  # [bad-string-format-type]
"%x" % 1.1  # [bad-string-format-type]
"%(key)x" % {"key": 1.1}  # [bad-string-format-type]
"%d" % []  # [bad-string-format-type]
"%(key)d" % {"key": []}  # [bad-string-format-type]

WORD = "abc"
"%d" % WORD  # [bad-string-format-type]
"%d %s" % (WORD, WORD)  # [bad-string-format-type]

VALUES_TO_FORMAT = (1, "2", 3.0)
"%d %s %f" % VALUES_TO_FORMAT
"%d %d %f" % VALUES_TO_FORMAT  # [bad-string-format-type]


def test_format(my_input_value, my_other_input_value):
    """In some cases it is not possible to determine the content of the input variable.
    Pylint must not crash or yield false positives in such cases.
    """
    print("%d" % my_input_value)
    print("%d %s" % (my_input_value, my_other_input_value))
    to_be_formatted = (my_input_value, my_other_input_value)
    print("%d %s" % to_be_formatted)



"""unused import"""
# pylint: disable=undefined-all-variable, import-error, too-few-public-methods, missing-docstring,wrong-import-position, multiple-imports
import xml.etree  # [unused-import]
import xml.sax  # [unused-import]
import os.path as test  # [unused-import]
from abc import ABCMeta
from sys import argv as test2  # [unused-import]
from sys import flags  # [unused-import]

# +1:[unused-import,unused-import]
from collections import deque, OrderedDict, Counter
import re, html.parser  # [unused-import]
import six

DATA = Counter()
# pylint: disable=self-assigning-variable
from fake import SomeName, SomeOtherName  # [unused-import]


class SomeClass:
    SomeName = SomeName  # https://bitbucket.org/logilab/pylint/issue/475
    SomeOtherName = 1
    SomeOtherName = SomeOtherName


from never import __all__

# pylint: disable=wrong-import-order,ungrouped-imports,reimported
import typing
from typing import TYPE_CHECKING
import typing as t


if typing.TYPE_CHECKING:
    import collections
if TYPE_CHECKING:
    import itertools
if t.TYPE_CHECKING:
    import xml


def get_ordered_dict() -> "collections.OrderedDict":
    return []


def get_itertools_obj() -> "itertools.count":
    return []


def use_html_parser() -> "html.parser.HTMLParser":
    return html.parser.HTMLParser


import os  # [unused-import]
import sys


class NonRegr:
    """???"""

    def __init__(self):
        print("initialized")

    def sys(self):
        """should not get sys from there..."""
        print(self, sys)

    def dummy(self, truc):
        """yo"""
        return self, truc

    def blop(self):
        """yo"""
        print(self, "blip")


if TYPE_CHECKING:
    if sys.version_info >= (3, 6, 2):
        from typing import NoReturn

# Pathological cases
from io import TYPE_CHECKING  # pylint: disable=no-name-in-module
import trace as t
import astroid as typing  # pylint: disable=shadowed-import

TYPE_CHECKING = "red herring"

if TYPE_CHECKING:
    import unittest  # [unused-import]
if t.TYPE_CHECKING:  # pylint: disable=no-member
    import uuid  # [unused-import]
if typing.TYPE_CHECKING:  # pylint: disable=no-member
    import warnings  # [unused-import]
if typing.TYPE_CHECKING_WITH_MAGIC:  # pylint: disable=no-member
    import compileall  # [unused-import]

TYPE_CHECKING = False
if TYPE_CHECKING:
    import zoneinfo


class WithMetaclass(six.with_metaclass(ABCMeta)):
    """Regression test for https://github.com/pylint-dev/pylint/issues/7506.

    Requires six."""


# Regression test for https://github.com/pylint-dev/pylint/issues/3765
# `unused-import` should not be emitted when a type annotation uses quotation marks
from typing import List


class Bee:
    def get_all_classes(self) -> "List[Bee]":
        pass



""" docstring for file property_pattern.py """
class PropertyPatterns:
    prop1 = property(lambda self: self._prop1*2, None, None, "property usage 1")

    @property
    def prop2(self):
        """property usage 2"""
        return self._prop2

    @prop2.setter
    def prop2(self, value):
        self._prop2 = value * 2

    def __init__(self):
        self._prop1=1
        self._prop2=2



""" Dictionaries are reversible starting on python 3.8"""
# pylint: disable=missing-docstring

# This can't be detected since changes to locals aren't backported
reversed({'a': 1, 'b': 2})


class InheritDict(dict):
    pass


reversed(InheritDict({'a': 1, 'b': 2})) # [bad-reversed-sequence]



class Soup:
    @staticmethod
    def temp():
        print("Soup is hot!")


class TomatoSoup(Soup):
    @staticmethod
    def temp():
        super().temp()
        print("But tomato soup is even hotter!")



"""test string format error"""
# pylint: disable=unsupported-binary-operation,line-too-long, consider-using-f-string

PARG_1 = PARG_2 = PARG_3 = 1


def pprint():
    """Test string format"""
    print("%s %s" % {'PARG_1': 1, 'PARG_2': 2})  # [too-few-format-args]
    print("%s" % (PARG_1, PARG_2))  # [too-many-format-args]
    print("%(PARG_1)d %d" % {'PARG_1': 1, 'PARG_2': 2})  # [mixed-format-string]
    print("%(PARG_1)d %(PARG_2)d" % {'PARG_1': 1})  # [missing-format-string-key]
    print("%(PARG_1)d %(PARG_2)d" % {'PARG_1': 1, 'PARG_2':2, 'PARG_3':3})  # [unused-format-string-key]
    print("%(PARG_1)d %(PARG_2)d" % {'PARG_1': 1, 2:3}) # [missing-format-string-key,bad-format-string-key]
    print("%(PARG_1)d %(PARG_2)d" % (2, 3)) # [format-needs-mapping]
    print("%(PARG_1)d %(PARG_2)d" % [2, 3]) # [format-needs-mapping]
    print("%2z" % PARG_1)  # [bad-format-character]
    print("strange format %2" % PARG_2) # [truncated-format-string]
    print("works in 3 %a" % 1)
    print("String" % PARG_1) # [format-string-without-interpolation]
    print("String" % ())  # [format-string-without-interpolation]
    print("String" % [])  # [format-string-without-interpolation]
    print("String" % None)  # [format-string-without-interpolation]



class Worm:  # [too-few-public-methods]
    def __init__(self, name: str, fruit_of_residence: Fruit):
        self.name = name
        self.fruit_of_residence = fruit_of_residence

    def bore(self):
        print(f"{self.name} is boring into {self.fruit_of_residence}")



# pylint: disable=missing-docstring,unused-import,import-error, wildcard-import,unused-wildcard-import
# pylint: disable=redefined-builtin,no-name-in-module,ungrouped-imports,wrong-import-order,wrong-import-position
# pylint: disable=consider-using-from-import

from time import sleep, sleep  # [reimported]
from lala import missing, missing  # [reimported]

import missing1
import missing1 # [reimported]

from collections import deque
from itertools import deque # [reimported]

from collections import OrderedDict
from itertools import OrderedDict as NotOrderedDict

from itertools import *
from os import *

import sys

import xml.etree.ElementTree
from xml.etree import ElementTree  # [reimported]

from email import encoders
import email.encoders  # [reimported]

import sys  # [reimported]  #pylint: disable=ungrouped-imports,wrong-import-order

def no_reimport():
    """docstring"""
    import os  #pylint: disable=import-outside-toplevel
    print(os)


def reimport():
    """This function contains a reimport."""
    import sys  # [reimported,redefined-outer-name] #pylint: disable=import-outside-toplevel
    del sys


del sys, ElementTree, xml.etree.ElementTree, encoders, email.encoders

from pandas._libs import algos as libalgos
import pandas._libs.algos as algos  # [reimported]



def even_number_under(n: int):  # [missing-yield-doc]
    """Prints even numbers smaller than n.
    Args:
        n: Upper limit of even numbers.
    """
    for i in range(n):
        if i % 2 == 1:
            continue
        yield i



"""Tests for undefined variable with assignment expressions"""
# pylint: disable=using-constant-test, expression-not-assigned

# Tests for annotation of variables and potentially undefinition

from typing import TYPE_CHECKING

def typing_and_assignment_expression():
    """The variable gets assigned in an assignment expression"""
    var: int
    if (var := 1 ** 2):
        print(var)


def typing_and_self_referencing_assignment_expression():
    """The variable gets assigned in an assignment expression that references itself"""
    var: int
    if (var := var ** 2):  # [used-before-assignment]
        print(var)


def self_referencing_assignment_expression():
    """An invalid self-referencing assignment expression"""
    if (var := var()):  # [used-before-assignment]
        print(var)


def typed_assignment_in_function_default(param: str = (typed_default := "walrus")) -> None:
    """An annotated assignment expression in a default parameter should not emit"""
    return param


print(typed_default)


def assignment_in_function_default(param = (default := "walrus")) -> None:
    """An assignment expression in a default parameter should not emit"""
    return param


print(default)


def no_assignment_in_function_default(param: str = "walrus") -> None:
    """No annotated assignment expression should emit"""
    return param


print(no_default) # [undefined-variable]


def no_parameters_in_function_default() -> None:
    """Regression tests for functions without any parameters"""
    return


print(again_no_default) # [undefined-variable]

# Tests for assignment expressions in if ... else comprehensions


[i for i in range(10) if (if_assign_1 := i)]

print(if_assign_1)

IF_TWO = [i for i in range(10) if (if_assign_2 := i)]

print(if_assign_2)

IF_THREE = next(i for i in range(10) if (if_assign_3 := i))

print(if_assign_3)

IF_FOUR = {i: i for i in range(10) if (if_assign_4 := i)}

print(if_assign_4)

IF_FIVE = {i: i if (if_assign_5 := i) else 0 for i in range(10)}
print(if_assign_5)

{i: i if True else (else_assign_1 := i) for i in range(10)}

print(else_assign_1) # [undefined-variable]


# Tests for assignment expressions in the assignment of comprehensions

[(assign_assign_1 := i) for i in range(10)]

print(assign_assign_1)

COMPREHENSION_TWO =[(assign_assign_2 := i) for i in range(10)]

print(assign_assign_2)

COMPREHENSION_THREE = next((assign_assign_3 := i) for i in range(10))

print(assign_assign_3)

COMPREHENSION_FOUR = {i: (assign_assign_4 := i) for i in range(10)}

print(assign_assign_4)

COMPREHENSION_FIVE = {i: (else_assign_2 := i) if False else 0 for i in range(10)}

print(else_assign_2)  # [undefined-variable]


# Tests for assignment expressions in lambda statements

things = []
sorted_things = sorted(
    things,
    key=lambda thing: x_0
    if (x_0 := thing.this_value) < (x_1 := thing.that_value)
    else x_1,
)


# Tests for type annotation reused in comprehension

def type_annotation_used_after_comprehension():
    """https://github.com/pylint-dev/pylint/issues/5326#issuecomment-982635371"""
    my_int: int
    ints = [my_int + 1 for my_int in range(5)]

    for my_int in ints:
        print(my_int)


def type_annotation_unused_after_comprehension():
    """https://github.com/pylint-dev/pylint/issues/5326"""
    my_int: int
    _ = [print(sep=my_int, end=my_int) for my_int in range(10)]


def type_annotation_used_improperly_after_comprehension():
    """https://github.com/pylint-dev/pylint/issues/5654"""
    my_int: int
    _ = [print(sep=my_int, end=my_int) for my_int in range(10)]
    print(my_int)  # [used-before-assignment]


def type_annotation_used_improperly_after_comprehension_2():
    """Same case as above but with positional arguments"""
    my_int: int
    _ = [print(my_int, my_int) for my_int in range(10)]
    print(my_int)  # [used-before-assignment]


# Tests for named expressions (walrus operator)

# Expression in ternary operator: positional argument
print(sep=colon if (colon := ":") else None)


class Dummy:
    """Expression in ternary operator: keyword argument"""
    # pylint: disable=too-few-public-methods
    def __init__(self, value):
        self.value = value


dummy = Dummy(value=val if (val := 'something') else 'anything')

def expression_in_ternary_operator_inside_container():
    """Named expression in ternary operator: inside container"""
    return [val2 if (val2 := 'something') else 'anything']


def expression_in_ternary_operator_inside_container_tuple():
    """Same case, using a tuple inside a 1-element list"""
    return [(val3, val3) if (val3 := 'something') else 'anything']


def expression_in_ternary_operator_inside_container_later_position():
    """
    Named expression follows unrelated item in container.

    If 23 is replaced with `val3`, there is currently a false negative,
    but the false positive here is more important and likely to occur."""
    return [23, val3 if (val3 := 'something') else 'anything']


# Self-referencing
if (z := z):  # [used-before-assignment]
    z = z + 1


if (defined := False):
    NEVER_DEFINED = 1
print(defined)
print(NEVER_DEFINED)  # [used-before-assignment]

if (still_defined := False) == 1:
    NEVER_DEFINED_EITHER = 1
print(still_defined)


if TYPE_CHECKING:
    import enum
    import weakref
elif input():
    if input() + 1:
        pass
    elif (enum := None):
        pass
    else:
        print(None if (weakref := '') else True)
else:
    pass

def defined_by_walrus_in_type_checking() -> weakref:
    """Usage of variables defined in TYPE_CHECKING blocks"""
    print(enum)
    return weakref



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

from __future__ import annotations

import warnings

import astroid
from astroid import nodes

from pylint.checkers.base_checker import BaseChecker
from pylint.checkers.utils import only_required_for_messages
from pylint.utils import ASTWalker


class TestASTWalker:
    class MockLinter:
        def __init__(self, msgs: dict[str, bool]) -> None:
            self._msgs = msgs

        def is_message_enabled(self, msgid: str) -> bool:
            return self._msgs.get(msgid, True)

    class Checker(BaseChecker):
        # pylint: disable-next=super-init-not-called
        def __init__(self) -> None:
            self.called: set[str] = set()

        @only_required_for_messages("first-message")
        def visit_module(
            self, module: nodes.Module  # pylint: disable=unused-argument
        ) -> None:
            self.called.add("module")

        @only_required_for_messages("second-message")
        def visit_call(self, module: nodes.Call) -> None:
            raise NotImplementedError

        @only_required_for_messages("second-message", "third-message")
        def visit_assignname(
            self, module: nodes.AssignName  # pylint: disable=unused-argument
        ) -> None:
            self.called.add("assignname")

        @only_required_for_messages("second-message")
        def leave_assignname(self, module: nodes.AssignName) -> None:
            raise NotImplementedError

    def test_only_required_for_messages(self) -> None:
        linter = self.MockLinter(
            {"first-message": True, "second-message": False, "third-message": True}
        )
        walker = ASTWalker(linter)  # type: ignore[arg-type]
        checker = self.Checker()
        walker.add_checker(checker)
        walker.walk(astroid.parse("x = func()"))
        assert {"module", "assignname"} == checker.called

    def test_deprecated_methods(self) -> None:
        class Checker(BaseChecker):
            # pylint: disable-next=super-init-not-called
            def __init__(self) -> None:
                self.called = False

            @only_required_for_messages("first-message")
            def visit_assname(
                self, node: nodes.AssignName  # pylint: disable=unused-argument
            ) -> None:
                self.called = True

        linter = self.MockLinter({"first-message": True})
        walker = ASTWalker(linter)  # type: ignore[arg-type]
        checker = Checker()
        walker.add_checker(checker)
        with warnings.catch_warnings(record=True):
            warnings.simplefilter("always")
            walker.walk(astroid.parse("x = 1"))

            assert not checker.called



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

"""This script updates towncrier.toml and creates a new newsfile and intermediate
folders if necessary.
"""
from __future__ import annotations

import argparse
import re
from pathlib import Path
from subprocess import check_call

NEWSFILE_PATTERN = re.compile(r"doc/whatsnew/\d/\d.\d+/index\.rst")
NEWSFILE_PATH = "doc/whatsnew/{major}/{major}.{minor}/index.rst"
TOWNCRIER_CONFIG_FILE = Path("towncrier.toml")
TOWNCRIER_VERSION_PATTERN = re.compile(r"version = \"(\d+\.\d+\.\d+)\"")

NEWSFILE_CONTENT_TEMPLATE = """
***************************
 What's New in Pylint {major}.{minor}
***************************

.. toctree::
   :maxdepth: 2

:Release:{major}.{minor}
:Date: TBA

Summary -- Release highlights
=============================


.. towncrier release notes start
"""


def main() -> None:
    parser = argparse.ArgumentParser()
    parser.add_argument("version", help="The new version to set")
    parser.add_argument(
        "--dry-run",
        action="store_true",
        help="Just show what would be done, don't write anything",
    )
    args = parser.parse_args()

    if "dev" in args.version:
        print("'-devXY' will be cut from version in towncrier.toml")
    match = re.match(
        r"^(0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*)(-?\w+\d*)*", args.version
    )
    if not match:
        print(
            "Fatal error - new version did not match the "
            "expected format (major.minor.patch[.*]). Abort!"
        )
        return
    major, minor, patch, suffix = match.groups()
    new_version = f"{major}.{minor}.{patch}"

    new_newsfile = NEWSFILE_PATH.format(major=major, minor=minor)
    create_new_newsfile_if_necessary(new_newsfile, major, minor, args.dry_run)
    patch_towncrier_toml(new_newsfile, new_version, args.dry_run)
    build_changelog(suffix, args.dry_run)


def create_new_newsfile_if_necessary(
    new_newsfile: str, major: str, minor: str, dry_run: bool
) -> None:
    new_newsfile_path = Path(new_newsfile)
    if new_newsfile_path.exists():
        return

    # create new file and add boiler plate content
    if dry_run:
        print(
            f"Dry run enabled - would create file {new_newsfile} "
            "and intermediate folders"
        )
        return

    print("Creating new newsfile:", new_newsfile)
    new_newsfile_path.parent.mkdir(parents=True, exist_ok=True)
    new_newsfile_path.touch()
    new_newsfile_path.write_text(
        NEWSFILE_CONTENT_TEMPLATE.format(major=major, minor=minor),
        encoding="utf8",
    )

    # tbump does not add and commit new files, so we add it ourselves
    print("Adding new newsfile to git")
    check_call(["git", "add", new_newsfile])


def patch_towncrier_toml(new_newsfile: str, version: str, dry_run: bool) -> None:
    file_content = TOWNCRIER_CONFIG_FILE.read_text(encoding="utf-8")
    patched_newsfile_path = NEWSFILE_PATTERN.sub(new_newsfile, file_content)
    new_file_content = TOWNCRIER_VERSION_PATTERN.sub(
        f'version = "{version}"', patched_newsfile_path
    )
    if dry_run:
        print("Dry run enabled - this is what I would write:\n")
        print(new_file_content)
        return
    TOWNCRIER_CONFIG_FILE.write_text(new_file_content, encoding="utf-8")


def build_changelog(suffix: str | None, dry_run: bool) -> None:
    if suffix:
        print("Not a release version, skipping changelog generation")
        return

    if dry_run:
        print("Dry run enabled - not building changelog")
        return

    print("Building changelog")
    check_call(["towncrier", "build", "--yes"])


if __name__ == "__main__":
    main()



"""Test detection of redundant nested calls to min/max functions"""

# pylint: disable=redefined-builtin,unnecessary-lambda-assignment

min(1, min(2, 3))  # [nested-min-max]
max(1, max(2, 3))  # [nested-min-max]
min(min(1, 2), 3)  # [nested-min-max]
min(min(min(1, 2), 3), 4)  # [nested-min-max, nested-min-max]
min(1, max(2, 3))
min(1, 2, 3)
min(min(1, 2), min(3, 4))  # [nested-min-max]
min(len([]), min(len([1]), len([1, 2])))  # [nested-min-max]

orig_min = min
min = lambda *args: args[0]
min(1, min(2, 3))
orig_min(1, orig_min(2, 3))  # [nested-min-max]

# This is too complicated (for now) as there is no clear better way to write it
max(max(i for i in range(10)), 0)
max(max(max(i for i in range(10)), 0), 1)

# These examples can be improved by splicing
lst = [1, 2]
max(3, max(lst))  # [nested-min-max]
max(3, *lst)

nums = (1, 2,)
max(3, max(nums))  # [nested-min-max]
max(3, *nums)

nums = {1, 2}
max(3, max(nums))  # [nested-min-max]
max(3, *nums)

nums = {1: 2, 7: 10}
max(3, max(nums))  # [nested-min-max]
max(3, *nums)

max(3, max(nums.values()))  # [nested-min-max]
max(3, *nums.values())

lst2 = [3, 7, 10]
max(3, max(nums), max(lst2))  # [nested-min-max]

max(3, max([5] + [6, 7]))  # [nested-min-max]
max(3, *[5] + [6, 7])

max(3, max([5] + [i for i in range(4) if i]))  # [nested-min-max]
max(3, *[5] + [i for i in range(4) if i])

max(3, max([5] + list(range(4))))  # [nested-min-max]
max(3, *[5] + list(range(4)))

max(3, max(list(range(4))))  # [nested-min-max]
max(3, *list(range(4)))

# Nesting is useful for finding the maximum in a matrix
# No message if external call has exactly 1 argument
matrix = [[1, 2, 3], [4, 5, 6]]
max(max(matrix))
max(max(max(matrix)))
max(3, max(max(matrix)))  # [nested-min-max]
max(max(3, max(matrix)))  # [nested-min-max]



"""Tests for redefining builtins."""
# pylint: disable=unused-import, wrong-import-position, reimported, import-error
# pylint: disable=redefined-outer-name, import-outside-toplevel, wrong-import-order


def function():
    """Redefined local."""
    type = 1  # [redefined-builtin]
    print(type)


# pylint:disable=invalid-name
map = {}  # [redefined-builtin]
__doc__ = "reset the doc"


# Test redefining-builtins
from notos import open  # [redefined-builtin]

# Test default redefining-builtins-modules setting
from os import open

# Test non-default redefining-builtins-modules setting in function
def test():
    """Function importing a function"""
    from os import open



"""Checks for unnecessary-dunder-call on __aiter__/__anext__ with py-version=3.9."""


class MyClass:
    """A class implementing __aiter__ and __anext__."""

    def __aiter__(self):
        ...

    async def __anext__(self):
        ...


MyClass().__aiter__()
MyClass().__anext__()



import random

MAX_NUM_OF_ITERATIONS = 5
THRESHOLD_VAL = 100
MIN_MEASUREMENT_VAL = 0
MAX_MEASUREMENT_VAL = 200

measurement = random.randint(MIN_MEASUREMENT_VAL, MAX_MEASUREMENT_VAL)
above_threshold = False
i = 0
while i < MAX_NUM_OF_ITERATIONS:
    above_threshold = measurement > THRESHOLD_VAL
    if above_threshold:
        break
    measurement = random.randint(MIN_MEASUREMENT_VAL, MAX_MEASUREMENT_VAL)



"""Test assignment expressions"""
# pylint: disable=missing-docstring,unused-argument,unused-import,invalid-name
# pylint: disable=blacklisted-name,unused-variable,pointless-statement,unused-variable
# pylint: disable=unnecessary-lambda-assignment

import re

if (a := True):
    x = a
else:
    x = False

x = b if (b := True) else False
x2: bool = b2 if (b2 := True) else False
x3 = 0
x3 += b3 if (b3 := 4) else 6

a = ["a   ", "b   ", "c   "]
c = [text for el in a if (text := el.strip()) == "b"]


# check wrong usage
assert err_a, (err_a := 2)  # [used-before-assignment]
print(err_b and (err_b := 2))  # [used-before-assignment]
values = (
    err_c := err_d,  # [used-before-assignment]
    err_d := 2,
)


# https://github.com/pylint-dev/pylint/issues/3347
s = 'foo' if (fval := lambda: 1) is None else fval


# https://github.com/pylint-dev/pylint/issues/3953
assert (n := 2) == 1, f"Expected 1, but got {n}"
dict({1: (o := 2)}, data=o)
assert (p := 2) == 1, \
    p

FOO_PATT = re.compile("")
foo = m.group("foo") if (m := FOO_PATT.match("")) else False


# https://github.com/pylint-dev/pylint/issues/3865
if (c := lambda: 2) and c():
    print("ok")

def func():
    print((d := lambda: 2) and d)


# https://github.com/pylint-dev/pylint/issues/3275
values = (
    e := 1,
    f := e,
)
print(values)

function = lambda: (
    h := 1,
    i := h,
)
print(function())


# https://github.com/pylint-dev/pylint/issues/3763
foo if (foo := 3 - 2) > 0 else 0


# https://github.com/pylint-dev/pylint/issues/4238
l1 = f'The number {(count1 := 4)} ' \
     f'is equal to {count1}'
l2: str = (
    f'The number {(count2 := 4)} '
    f'is equal to {count2}'
)
l3 = "Hello "
l3 += (
    f'The number {(count3 := 4)} '
    f'is equal to {count3}'
)


# https://github.com/pylint-dev/pylint/issues/4301
def func2():
    return f'The number {(count := 4)} ' \
           f'is equal to {count}'


# https://github.com/pylint-dev/pylint/issues/4828
def func3():
    return bar if (bar := "") else ""


# Lambda and IfExp
def func4():
    l = lambda x: y if (y := x) else None


# Crash related to assignment expression in nested if statements
# See https://github.com/pylint-dev/pylint/issues/5178
def func5(val):
    variable = None

    if val == 1:
        variable = "value"
        if variable := "value":
            pass

    elif val == 2:
        variable = "value_two"
        variable = "value_two"

    return variable



# pylint: disable=missing-docstring,too-few-public-methods
class Entity[_T: float]:
    last_update: int | None = None

    def __init__(self, data: _T) -> None:  # [undefined-variable]  # false-positive
        self.data = data


class Sensor(Entity[int]):
    def __init__(self, data: int) -> None:
        super().__init__(data)

    def async_update(self) -> None:
        self.data = 2

        if self.last_update is None:
            pass
        self.last_update = 2


class Switch(Entity[int]):
    def __init__(self, data: int) -> None:
        Entity.__init__(self, data)


class Parent[_T]:
    def __init__(self):
        self.update_interval = 0


class Child[_T](Parent[_T]):  # [undefined-variable]  # false-positive
    def func(self):
        self.update_interval = None



from _monkey import Monkey


class Tree:
    def __init__(self):
        self.number_of_bananas = 5
        self.inhabitant = Monkey(
            "Steve"
        )  # This will trigger the evaluation of `monkey.py`



class Person:
    __slots__ = ("_age", "name")

    def __init__(self, age, name):
        self._age = age
        self.name = name

    @property
    def age(self):
        return self._age

    def say_hi(self):
        print(f"Hi, I'm {self.name}.")



"""Tests for missing-raises-doc for exception class inheritance."""
# pylint: disable=missing-class-docstring

class CustomError(NameError):
    pass


class CustomChildError(CustomError):
    pass


def test_find_missing_raise_for_parent():  # [missing-raises-doc]
    """This is a docstring.

    Raises:
        CustomError: Never
    """
    raise NameError("hi")


def test_no_missing_raise_for_child_builtin():
    """This is a docstring.

    Raises:
        Exception: Never
    """
    raise ValueError("hi")


def test_no_missing_raise_for_child_custom():
    """This is a docstring.

    Raises:
        NameError: Never
    """
    raise CustomError("hi")


def test_no_missing_raise_for_child_custom_nested():
    """This is a docstring.

    Raises:
        NameError: Never
    """
    raise CustomChildError("hi")



class Fruit:
    FRUITS = {"apple": "red", "orange": "orange"}

    def __print_color(self, name, color):
        print(f"{name}: {color}")

    def print(self):
        for fruit, color in self.FRUITS.items():
            self.__print_color(fruit, color)



"""Checks of Dosctrings 'docstring-first-line-empty'"""
# pylint: disable=too-few-public-methods,bad-docstring-quotes

def check_messages(*messages):  # [docstring-first-line-empty]
    """
    docstring"""
    return messages


def function2():
    """Test Ok"""


class FFFF:  # [docstring-first-line-empty]
    """
    Test Docstring First Line Empty
    """

    def method1(self):  # [docstring-first-line-empty]
        '''
        Test Triple Single Quotes docstring
        '''



class Person:
    # +1: [class-variable-slots-conflict, class-variable-slots-conflict, class-variable-slots-conflict]
    __slots__ = ("age", "name", "say_hi")
    name = None

    def __init__(self, age, name):
        self.age = age
        self.name = name

    @property
    def age(self):
        return self.age

    def say_hi(self):
        print(f"Hi, I'm {self.name}.")



"""Test typing.TypedDict"""
# pylint: disable=invalid-name,missing-class-docstring,pointless-statement
import typing
from typing import TypedDict


class CustomTD(TypedDict):
    var: int


class CustomTD2(typing.TypedDict, total=False):
    var2: str


class CustomTD3(CustomTD2):
    var3: int


CustomTD4 = TypedDict("CustomTD4", var4=bool)

CustomTD5 = TypedDict("CustomTD5", {"var5": bool})

my_dict = CustomTD(var=1)
my_dict["var"]
my_dict["var"] = 2


# https://github.com/pylint-dev/pylint/issues/4715
# Instance of TypedDict should be callable
Link = TypedDict("Link", {"href": str})
Link(href="foo")



# This is not a standalone test
# Monkey class is called from Tree class in delayed_external_monkey_patching.py


class Monkey:
    def __init__(self, name):
        # pylint: disable=import-outside-toplevel
        from delayed_external_monkey_patching import Tree

        self.name = name
        self.tree = Tree()
        self.tree.has_tasty_bananas = True  # This monkey patching will increase the number of items in instance_attrs for `Tree`



""" Checks that reversed() receive proper argument """
# pylint: disable=missing-docstring,invalid-name,unused-variable
# pylint: disable=too-few-public-methods


def test():
    def parent():
        a = 42

        def stuff():
            nonlocal a

    c = 24

    def parent2():
        a = 42

        def stuff():
            def other_stuff():
                nonlocal a
                nonlocal c


b = 42


def func():
    def other_func():
        nonlocal b  # [nonlocal-without-binding]

    # Case where `nonlocal-without-binding` was not emitted when
    # the nonlocal name was assigned later in the same scope.
    # https://github.com/pylint-dev/pylint/issues/6883
    def other_func2():
        nonlocal c  # [nonlocal-without-binding]
        c = 1


class SomeClass:
    nonlocal x  # [nonlocal-without-binding]

    def func(self):
        nonlocal some_attr  # [nonlocal-without-binding]


def func2():
    nonlocal_ = None
    local = None

    class Class:

        nonlocal nonlocal_

        nonlocal_ = 1
        local = 1

    return local + nonlocal_


def function():
    """Test for `unused-variable` when multiple-assignment contains a `nonlocal`"""
    myint, mylist = 0, []

    print(mylist)

    def inner():
        nonlocal myint
        mylist.append(myint)
        myint += 1

    return inner()


nonlocal APPLE  # [nonlocal-without-binding]
APPLE = 42



"""Check unknown or deleted option. """

# Standard disable with unknown value
# pylint: disable=C05048  # [unknown-option-value]
# Standard disable with deleted symbol
# pylint: disable=execfile-builtin  # [useless-option-value]
# Standard disable with symbol moved to extension
# pylint: disable=no-self-use  # [useless-option-value]
# Standard disable with deleted msgid
# pylint: disable=W1656  # [useless-option-value]
# Standard disable with msgid moved to extension
# pylint: disable=R0201  # [useless-option-value]
# disable-next with unknown value
# pylint: disable-next=R78948  # [unknown-option-value]
# disable-next with deleted symbol
# pylint: disable-next=deprecated-types-field  # [useless-option-value]
# disable-next with deleted msgid
# pylint: disable-next=W1634  # [useless-option-value]
# disable-next with symbol moved to extension
# pylint: disable-next=no-self-use  # [useless-option-value]
# disable-next with msgid moved to extension
# pylint: disable-next=R0201  # [useless-option-value]

# enable with unknown value
# pylint:enable=W04044  # [unknown-option-value]
# enable with deleted symbol
# pylint:enable=dict-values-not-iterating  # [useless-option-value]
# enable with symbol moved to extension
# pylint: enable=no-self-use  # [useless-option-value]
# enable with deleted msgid
# pylint:enable=W1622  # [useless-option-value]
# enable with msgid moved to extension
# pylint: enable=R0201  # [useless-option-value]

# Standard disable with deleted old name symbol of deleted message
# pylint: disable=no-space-after-operator  # [useless-option-value]
# Standard disable with deleted old name msgid of deleted message
# pylint: disable=C0323  # [useless-option-value]



def three_d_chess_move(  # [too-many-arguments]
    x_white,
    y_white,
    z_white,
    piece_white,
    x_black,
    y_black,
    z_black,
    piece_black,
    x_blue,
    y_blue,
    z_blue,
    piece_blue,
    current_player,
):
    pass



# pylint: disable=missing-docstring, comparison-with-itself


if 2 is 2:  # [literal-comparison, comparison-of-constants]
    pass

if "a" is b"a":  # [literal-comparison, comparison-of-constants]
    pass

if 2.0 is 3.0:  # [literal-comparison, comparison-of-constants]
    pass

if () is (1, 2, 3):
    pass

if () is {1:2, 2:3}:  # [literal-comparison]
    pass

if [] is [4, 5, 6]:  # [literal-comparison]
    pass

if () is {1, 2, 3}:  # [literal-comparison]
    pass

if () is not {1:2, 2:3}: # [literal-comparison]
    pass

if [] is not [4, 5, 6]: # [literal-comparison]
    pass

if () is not {1, 2, 3}: # [literal-comparison]
    pass


CONST = 24


if CONST is 0: # [literal-comparison]
    pass

if CONST is 1: # [literal-comparison]
    pass

if CONST is 42: # [literal-comparison]
    pass

# We do not do inference for this check, since the comparing
# object might be used as a sentinel.
if () is CONST:
    pass

def github_issue_3031(arg=()):
    if arg is ():
        pass

    if arg is not ():
        pass



# pylint: disable=missing-docstring,too-few-public-methods

class Meta(type):
    @property
    def values(cls):
        return ['foo', 'bar']


class Parent(metaclass=Meta):
    pass


assert 'foo' in Parent.values  # no warning
for value in Parent.values:  # no warning
    print(value)


class Child(Parent):
    pass


assert 'foo' in Child.values  # false-positive: unsupported-membership-test
for value in Child.values:  # false-positive: not-an-iterable
    print(value)


class Meta2(type):
    def a_method(cls):
        return [123]


class Parent2(metaclass=Meta2):
    @property
    def a_method(self):
        return "actually a property"


class Child2(Parent2):
    pass


assert 123 in Child2.a_method  # [unsupported-membership-test]
for value in Child2.a_method:  # [not-an-iterable]
    print(value)



# Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt

"""Test that the directory structure of the functional tests is correct."""
from pathlib import Path

from pylint.testutils.functional.find_functional_tests import (
    get_functional_test_files_from_directory,
)


def test_directories() -> None:
    """Test that the directory structure of the functional tests is correct."""
    functional_dir = Path(__file__).parent / "functional"
    get_functional_test_files_from_directory(functional_dir)



def two_good_fruits_generator(fruits):
    for fruit in fruits:
        if not fruit.is_tasty():
            continue
        while True:
            next_fruit = next(fruits)  # [stop-iteration-return]
            if next_fruit.is_tasty():
                yield fruit, next_fruit
                break



# pylint: disable=missing-docstring,too-few-public-methods
"""#3291"""


class Myarray:
    def __init__(self, array):
        self.array = array

    def __mul__(self, val):
        return Myarray(val)

    def astype(self):
        return "ASTYPE", self

def randint(maximum):
    if maximum is not None:
        return Myarray([1, 2, 3]) * 2

    return int(5)

print(randint(1).astype()) # we don't want an error for astype access



class Fruit:
    def __init__(self) -> None:
        self.name = "apple"

    def __eq__(self, other: object) -> bool:
        return isinstance(other, Fruit) and other.name == self.name

    def __hash__(self) -> int:
        return hash(self.name)



class Animal:
    def run(self, distance=0):
        print(f"Ran {distance} km!")


class Dog(Animal):
    def run(self, distance=0):
        super(Animal, self).run(distance)
        print("Fetched that stick, wuff !")



""" Functional tests for method deprecation. """
# pylint: disable=missing-docstring, super-init-not-called, not-callable, comparison-of-constants
import base64
import inspect
import logging
import nntplib
import time
import unittest
import xml.etree.ElementTree

class MyTest(unittest.TestCase):
    def test(self):
        self.assert_(True)  # [deprecated-method]

xml.etree.ElementTree.Element('tag').getchildren()  # [deprecated-method]
xml.etree.ElementTree.Element('tag').getiterator()  # [deprecated-method]
nntplib.NNTP(None).xpath(None) # [deprecated-method]


inspect.getargspec(None) # [deprecated-method]
logging.warn("a") # [deprecated-method]
base64.encodestring("42") # [deprecated-method]
base64.decodestring("42") # [deprecated-method]


class SuperCrash(unittest.TestCase):

    def __init__(self):
        # should not crash.
        super()()

xml.etree.ElementTree.iterparse(None)


class Tests(unittest.TestCase):

    def test_foo(self):
        self.assertEquals(2 + 2, 4)  # [deprecated-method]
        self.assertNotEquals(2 + 2, 4)  # [deprecated-method]
        self.assertAlmostEquals(2 + 2, 4)  # [deprecated-method]
        self.assertNotAlmostEquals(2 + 2, 4)  # [deprecated-method]
        self.assert_("abc" == "2")  # [deprecated-method]

        self.assertRaisesRegex(ValueError, "exception")
        self.assertRegex("something", r".+")


class Deprecated:  # pylint: disable=too-few-public-methods
    deprecated_method = logging.warn


d = Deprecated()
d.deprecated_method()  # [deprecated-method]

def test(clock = time.time):
    """time.clock is deprecated but time.time via an alias is not!"""
    clock()



"""Tests for missing-return-doc and missing-return-type-doc for Sphinx style docstrings"""
# pylint: disable=function-redefined, invalid-name, undefined-variable, missing-function-docstring
# pylint: disable=unused-argument, disallowed-name, too-few-public-methods, missing-class-docstring
# pylint: disable=unnecessary-pass, use-yield-from
import abc


def my_func(self):
    """find_sphinx_returns

    :return: Always False
    :rtype: bool
    """
    return False


def my_func(self, doc_type):
    """ignores_sphinx_return_none

    :param doc_type: Sphinx
    :type doc_type: str
    """
    return


def my_func(self):
    """finds_sphinx_return_custom_class

    :returns: An object
    :rtype: :class:`mymodule.Class`
    """
    return mymodule.Class()


def my_func(self):
    """finds_sphinx_return_list_of_custom_class

    :returns: An object
    :rtype: list(:class:`mymodule.Class`)
    """
    return [mymodule.Class()]


def my_func(self):  # [redundant-returns-doc]
    """warns_sphinx_redundant_return_doc

    :returns: One
    """
    return None


def my_func(self):  # [redundant-returns-doc]
    """warns_sphinx_redundant_rtype_doc

    :rtype: int
    """
    return None


def my_func(self):
    """ignores_sphinx_redundant_return_doc_multiple_returns

    :returns: One
    :rtype: int

    :returns: None sometimes
    :rtype: None
    """
    if a_func():
        return None
    return 1


def my_func_with_yield(self):
    """ignore_sphinx_redundant_return_doc_yield

    :returns: One
    :rtype: generator
    """
    for value in range(3):
        yield value


class Foo:
    """test_ignores_return_in_abstract_method_sphinx
    Example of an abstract method documenting the return type that an
    implementation should return.
    """

    @abc.abstractmethod
    def foo(self):
        """docstring ...

        :returns: Ten
        :rtype: int
        """
        return 10


class Foo:
    def test_ignores_ignored_argument_names_sphinx(self, arg, _):
        """Example of a method documenting the return type that an
        implementation should return.


        :param arg: An argument.
        :type arg: int
        """
        pass



"""Tests for used-before-assignment with functions added in python 3.7"""
# pylint: disable=missing-function-docstring
from __future__ import annotations
from collections import namedtuple
from typing import List


class MyClass:
    """With the future import only default values can't refer to the base class"""

    def correct_typing_method(self, other: MyClass) -> bool:
        return self == other

    def second_correct_typing_method(self, other: List[MyClass]) -> bool:
        return self == other[0]

    def incorrect_default_method(
        self, other=MyClass() # [undefined-variable]
    ) -> bool:
        return self == other

    def correct_string_typing_method(self, other: "MyClass") -> bool:
        return self == other

    def correct_inner_typing_method(self) -> bool:
        def inner_method(self, other: MyClass) -> bool:
            return self == other

        return inner_method(self, MyClass())


class NamedTupleSubclass(namedtuple("NamedTupleSubclass", [])):
    """Taken from https://github.com/pylint-dev/pylint/issues/5982"""
    def method(self) -> NamedTupleSubclass:
        """Variables checker crashed when astroid did not supply a lineno"""



"""Test deprecated abc decorators from Python 3.3."""
# pylint: disable=missing-class-docstring,too-few-public-methods,missing-function-docstring,no-member

import abc

class MyClass:
    @abc.abstractclassmethod  # [deprecated-decorator]
    def my_method(cls):
        pass

class Foo:
    def __init__(self):
        self._baz = 84

    def method(self):
        return self._baz

    @method.setter          # Invalid decorator
    def method(self, value):
        self._baz = value



"""Test that versions below Py3.10 will not emit useless-suppression for
disabling deprecated-method (on a method deprecated in Py3.10.

This test can be run on all Python versions, but it will lack value when
Pylint drops support for 3.9."""
# pylint: disable=import-error, unused-import

import threading.current_thread  # pylint: disable=deprecated-method
